#include "TepEditClue.h"

/* Programmer Name: Nathaniel Cleveland
 * Date Created: Feb 20th, 2001
 * Program Updated by:
 *
 * Program Description:			Loops through and fills a linked list of stuff.
 *
 * Program dependacies:			fnDefToHex, fnEditChunker
 * Takes:						TepClueData **ppListRoot,	the List Root which I'll link to
 *								char pFileToOpen[80],		the file name i want to open
 * Gives:						The BOOL value FAILURE (-1), if it doesn't work
 *								BOOL value TRUE, if it does.
 */
BOOL fnTepEditClue(TepClueData **ppListRoot, char pFileToOpen[80])
{
	TepClueData	*pCurrNode = *ppListRoot;				/* point to the first item in the list*/
    TepClueData	*pNewNode = NULL;						/* pointer to the newly created node */
	FILE		*FileHandle = NULL;						/* handle to the file I'm going to open */
	BOOL		bErrorCheck = TRUE;						/* varible to hold bool results */
	char		cpString[ARRAYSIZE];					/* string filled with file chunks */
	char		cpMemString[LONG_ARRAYSIZE];			/* used for long strings that aren't appreate for an array */

	memset(cpMemString, 0, LONG_ARRAYSIZE);				/* zero out the string(s) */
	memset(cpString, 0, ARRAYSIZE );

	/********************************************************************************/
	/**** 1. malloc mem for the list root, fail if we can't CALLER NOW DOES THIS ****/
	/********************************************************************************/

	/****************************************************************************/
	/**** 2. open the file, do a quick read to make sure the file is there ******/
	/****************************************************************************/
	if( (FileHandle  = fopen(pFileToOpen, "r" )) == NULL )
	{
      printf("Failed to open the file\n");
	  return FAILURE;
	}

	/*********************************/
	/**** 3. fill in the stuct(s) ****/
	/*********************************/
	/* loop thru and fill the structs. Stop filling whenever we encounter the string END OF FILE */
	for( ; ; )
	{

		pNewNode = calloc(1, sizeof(TepClueData));					/* allocate memory for the new node */

		if(pNewNode == NULL)										/* check for failure */
		{
			printf("failed to calloc memory\n");
			return FAILURE;
		}

		pNewNode->NEXT = NULL;										/* set the next node pointer to NULL */

		if(fnEditChunker(cpString, FileHandle, '\t') == FALSE)		/* get the next chunk of info, check for END OF FILE */
		{
			printf("break\n");
			break;
		}/* end if */

//		printf("We have this chunk: %s\n", cpString);				/* see what we have */
		/* fill in the id, check for the use of #defs */
		if(atoi(cpString) == 0)										/* check for string or number */
		{															/*  */
			pNewNode->id = fnDefToHex(cpString);					/* find the #def name given via deftohex */
			memset(cpString, 0, strlen(cpString) );					/* zero the string */
			bErrorCheck = fnEditChunker(cpString, FileHandle, '\t');/* chunk out the unique number */
			pNewNode->id = pNewNode->id + atoi(cpString);			/* Add the unique number */
		}/* end if */
		else														/* If it is just a number use that */
		{
			pNewNode->id = atoi(cpString);
		}/* end else */

//		printf("ID is %x\n", pNewNode->id);							/* read what we got */
		memset(cpString, 0, strlen(cpString) );						/* zero out the string */

		/* fill in the attachedto member */
		pNewNode->attachedto = calloc(1, sizeof(TepIDUnNode));		/* calloc mem for the pListRoot */

		if(pNewNode->attachedto == NULL)							/* check for error in calloc */
		{
			printf("calloc didn't alloc mem for the first attachedto.\n");
			return FAILURE;
		}

		if(fnTepEditUnNodeLink(&pNewNode->attachedto, FileHandle, '\t') == FALSE)
		{
			return FAILURE;
		}

		/* get details chunk */
		if(fnEditChunker(cpMemString, FileHandle, '\n') == FALSE)		/* get the next chunk of info, check for END OF FILE */
		{
			break;
		}

		/* fill in the details */
		pNewNode->details = calloc( strlen(cpMemString) + 1, sizeof(char));	/* calloc mem for the string */

		if(pNewNode->details == NULL)										/* check for error in calloc */
		{
			printf("calloc didn't alloc mem for the details.\n");
			return FAILURE;
		}
											/* copy the string to the memory alloced string */
		pNewNode->details = memmove(pNewNode->details, cpMemString, strlen(cpMemString) );
//		printf("Details are: %s", pNewNode->details);						/* see what we have */
		/* Test the node(s) */
/*
		printf("0x%x\n", pNewNode->id);
//		printf("0x0%x\n", pNewNode->attachedto);
		printf("%s\n", pNewNode->details);
*/
		/* set this node to the list */
		pCurrNode->NEXT = pNewNode;

		/* point CurrNode at this node */
		pCurrNode = pNewNode;
	}/* end for */


	if( fclose(FileHandle) != 0)
	{
		printf("damn file wouldn't close");
	}

	return TRUE;
}/* End fnTepEditDeath */

/****************** DEBUG MAIN ******************/
/*
int main(void)
{
	char inputArray[255];
	BOOL bCheckBool;
	TepClueData *pListRoot=NULL;

	pListRoot = calloc(1, sizeof(TepChar));

	while(1)
	{
		if(gets(inputArray) == NULL)
			break;

		if(inputArray == 0)
			break;

		bCheckBool = fnTepEditClue(&pListRoot, "ClueContent.txt");
		if(bCheckBool == FALSE)
		{
			printf("\nFailed\n");
		}
		else
		{
			printf("\nDidn't fail\n");
		}
	}
	free(pListRoot);
	return 0;
}
*/

int main ( int argc, char *argv[], char *envp[])
{
    pTepClueData pListRoot = NULL;

    if (argc > 1)
    {
        BOOL bCheckBool;

        pListRoot = calloc(1, sizeof(TepClueData));
        bCheckBool = fnTepEditClue(&pListRoot, argv[1]);

        if(bCheckBool == FAILURE || bCheckBool == FALSE)
        {
            printf("\nFailed\n");
        }
        else
        {
            printf("\nDidn't fail\n");
        }
    }
    else
    {
        printf("Usage:\n  TepEditClue <filename> [outputfile]\n    Where <filename> is the name of the file to be processed.\n    Outputfile is the name of the file to write to.\n\n");
    }

    if (pListRoot != NULL)
    {
        FILE *ioStream = NULL;

        if (argc > 2)
        {
            ioStream = fopen(argv[2], "w" );
        }
        else
        {
            ioStream = fopen("tepClue.dat", "w");
        }

        if (ioStream != NULL)
        {
            fnTepEditWriteFile(TEP_CLUE, pListRoot->NEXT, ioStream);
            fclose(ioStream);
        }
        else
        {
            printf("There was an error in opening the output file.\n");
        }

        //fnTepCharDelete(&pListRoot, NULL);
    }

    return 0;
}
