#include "tep_incs.h"

/*FUNCTION:         fnShowResponseSearch
 *PARAMETERS:       TepConvData *pListRoot      pointer to the start of a character's list
 *RETURN VALUES:    pointer the the node that matches the search
 *SIDE EFFECTS:     none
 *DESCRIPTION:  This funciton searches through a characters data to find the proper response
 *              to being shown evidence.
*/
TepConvData * fnShowReponseSearch(TepConvData *pListRoot, TepID bitMask)
{
    TepConvData *pCurrNode; /*Used to point at current node in search*/
    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        /*Check to see if the current node is the type of response we need*/
        if((pCurrNode->id & bitMask) == bitMask)
        {
            return pCurrNode;
        }/*end if*/
    }/*end for*/
    return NULL;
}/*end TepConvData*/

/*FUNCTION:         fnShow
 *PARAMETERS:       TepCurState *State  pointer to the current game state
 *RETURN VALUES:    SUCCESS or FAILURE
 *SIDE EFFECTS:     Adjusts the current characters whack value
 *                  Prints a response to the screen
 *DESCRIPTION:  This function is used to show a character evidence. It runs through
 *              a calculation based on the character's trust factor towards the player,
 *              the character's suspiscion index and the number of true and false clues
 *              shown to the character and adjusts the characters suspicion index accordingly.
*/
BOOL fnShow(TepCurState *State, TepParse msg)
{
    TepCurMurder       *pCurMurder = NULL;     /*Matt - pointer to the current days murder */
    TepPlayerEvidence  *pCurrNode = NULL;      /*Pointer for linked list search*/
    TepPlayerEvidence  *pEvidenceList = NULL;  /*ditto*/
    TepClueData        *pClueList = NULL;      /*ditto*/
    TepWhack           *pWhackList;            /*ditto*/
    TepChar            *pDude;                 /*Pointer to character being talked to*/
    TepWhack           *pAccused;              /*Pointer to the accused*/
    TepConvData        *pResponse;             /*Pointer to appropriate reponse*/
    int iHerrings = 0;                         /*Number of Red Herrings*/
    int iClues = 0;                            /*Number of Clues*/
    int iEvidence = 0;                         /*Amount of evidence */
    int iClueFactor = 0;                       /*+or-Clues - Red Herrings*/
    int iWhackAvg = 0;                         /*Average of suspicion list*/
    int iWhackSum = 0;                         /*used to calculate average*/
    int iNumChars = 0;                         /*used for calculations*/
    int iPlace = 0;                            /*ditto*/
    int iValue = 0;                            /*ditto*/
    int iWhackFactor = 1;                      /*used to determine whack modifier*/
    int iWhackModifier = 0;                    /*TrustFactor + WhackFactor + ClueFactor*/

	{
		TepPlayerEvidence *pEvList = State->player->evidenceGeneric;
		if (msg.Char2 != PARSE_UNUSED)
		{
			while (pEvList != NULL)
			{
				if (pEvList->attachedTo == msg.Char2)
				{
					break;
				}
				pEvList = pEvList->NEXT;
			}
			/* it didn't find the character */
			if (pEvList == NULL)
			{
				/* Jon - maybe we should say some kind of alert that the player has no evidence against the character specified */
				printf("You don't have any evidence against that villager.\n");
				return SUCCESS;
			}
		}
		else
		{
			/* Jon - maybe we should say some kind of alert that the player didnt specify a target */
			return SUCCESS;
		}
	}

    /* Matt - get the current murder */
    pCurMurder = State->currentMurder;
    if( pCurMurder != NULL )
    {
        while( pCurMurder->NEXT != NULL )
            pCurMurder = pCurMurder->NEXT;
    }

    /*Access evidence lists and count # of true clues, misplaced clues
      and red herrings*/
    pEvidenceList = State->player->evidenceGeneric; /* Matt - was evidenceAgainst, but everything is now stored in one list.. my fault */
    pClueList = pCurMurder->death->clues;/* Matt - this only grabs the first murder comitted. now it's a list so we have to grab the last 'TepCurMurder' in the list. Our game sucks. */

    for(pCurrNode = pEvidenceList; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if(fnClueDataSearch(pClueList, pCurrNode->clue, 0) != NULL)
        {
            iClues++;
        }
        iEvidence++;
    }/*end clue count*/

    /*The number of RedHerrings is equal to the amount of clues
    in evidence minus the number of Actual clues*/
    iHerrings = iEvidence - iClues;
    if(pCurMurder->murderer != msg.Char2)
    {
        iClues = -iClues;
    }
    iClueFactor = iClues - iHerrings;

    /*Obtain a pointer to the current character*/
    if((pDude = fnTepCharSearch(State->liveList, msg.Char1)) == NULL)
    {
		printf("\nCouldn't find pointer to the character\n");
        return FAILURE;
    }

/*WHACK FACTOR CALCULATIONS*/
    /*Find the whack list average*/
    for(pWhackList = pDude->whacklist; pWhackList != NULL; pWhackList = pWhackList->NEXT)
    {
        iWhackSum+= (int)pWhackList->value;
        iNumChars++;
        if(pWhackList->character == msg.Char2)
        {
            /*Find the suspicion index of the accused*/
            iValue = pWhackList->value;
            /*While we're at it, find a pointer to the accused*/
            pAccused = pWhackList;

        }
    }
    iWhackAvg = iWhackSum / iNumChars;

    /*Determine the number of characters above or below the mean and where the accused falls
      in the list*/
    iNumChars = 0;
    for(pWhackList = pDude->whacklist; pWhackList != NULL; pWhackList = pWhackList->NEXT)
    {
        if((iWhackFactor == 1) && (pWhackList->value > iWhackAvg))
        {
            iNumChars++;
            if(iValue >= pWhackList->value)
            {
                iPlace++;
            }
        }
        else if((iWhackFactor == -1) && ( pWhackList->value < iWhackAvg))
        {
            iNumChars++;
            if(iValue <= pWhackList->value)
            {
                iPlace++;
            }
        }
    }/*end for*/

    /*Determine the whack factor*/
    iWhackFactor = ((WHACKPOINTS / iNumChars) * iPlace);
    if(iValue < iWhackAvg)
    {
        iWhackFactor = -iWhackFactor;
        iWhackModifier = (int)pDude->like + iWhackFactor + CLUEMODIFIER * iClueFactor;
    }
    else
    {
        if((iWhackModifier = (int)pDude->like + iWhackFactor + CLUEMODIFIER * iClueFactor) < 0)
            iWhackModifier = 0;
    }

    /*Modifiy The Whack List*/
    pAccused->value+=iWhackModifier;

    /*Generate Responses*/
    if(iWhackModifier < -7)
    {
        /*Generate I really don't believe you*/
        if((pResponse = fnTepConvDataSearch(pDude->conv, 0, TEP_TALK | TALK_ANSEVIDENCEHELLNO)) != NULL)
        {
            fnPrintOut(NOPAUSE, NULL,NULL,1, pResponse->details);
        }
        else
        {	
			printf("\nI really don't believe you...\n");
            return FAILURE;
        }
    }/*end if*/
    else if(iWhackModifier < 0)
    {
        /*Generate I kinda don't believe you*/
        if((pResponse = fnTepConvDataSearch(pDude->conv, 0, TEP_TALK | TALK_ANSEVIDENCENO)) != NULL)
        {
            fnPrintOut(NOPAUSE, NULL,NULL,1, pResponse->details);
        }
        else
        {
			printf("\nI kinda don't believe you...\n");
            return FAILURE;
        }
    }/*end else if*/
    else if(iWhackModifier > 10)
    {
        /*Generate I really believe you*/
        if((pResponse = fnTepConvDataSearch(pDude->conv, 0, TEP_TALK | TALK_ANSEVIDENCEHELLYES)) != NULL)
        {
            fnPrintOut(NOPAUSE, NULL,NULL,1, pResponse->details);
        }
        else
        {
			printf("\nI really believe you...\n");
            return FAILURE;
        }
    }/*end else if*/
    else
    {
        /*Generate I kinda believe you*/
        if((pResponse = fnTepConvDataSearch(pDude->conv, TEP_TALK | TALK_ANSEVIDENCEYES, 0)) != NULL)
        {
            fnPrintOut(NOPAUSE, NULL,NULL,1, pResponse->details);
        }
        else
        {
			printf("\nI kinda believe you...\n");
            return FAILURE;
        }
    }/*end else*/
    return SUCCESS;
}/*end fnShow*/