#include "TepEditUtils.h"
#include "tep_constants.h"
#include "tep_lists.h"
#include "RoomDefines.h"
#include "tep_types.h"
#include "tepkeywordgenerator.h"

/* Programmer Name: Nathaniel Cleveland
 * Date Created: Feb 20th, 2001
 * Program Updated by:          Jon Olson - Feb 25, 2001 - Added saving functionality
 *
 * Program Description:         A collection of utilites for The TepEdit functionettes.
 *
 * Program dependacies:         CommandDefines.h, CharacterDefines.h
 */



/*FUNCTION:     fnEditChunker
 *PARAMETERS:   char *cpString              the string you want filled
 *              FILE *FileHandle            the file you want to look thru
 *              char cTerminator            what character you want to signafy the end of your string
 *RETURN VALUE: BOOLEAN TRUE                It did what it needed to do
 *              BOOLEAN FALSE               It found the END OF STRING
 *SIDE EFFECTS: moves the file pointer to the file handle that is passed
 *DESCRIPTION:  This function hacks through and pulls a string from a file, it'll stop when it sees the cTerminator
 *              character.
*/
BOOL fnEditChunker(char *cpString, FILE *FileHandle, char cTerminator)
{
    int     i = 0;                          /* counter */
    char    cComp = '\0';                   /* compairson checker */


    /* loop through and get the id */
    /* while the current char isn't a tab, keep filling the string */
    /* get the next character, inc the i */
    for(i = 0, cComp = getc(FileHandle) ; cComp != cTerminator ; cComp = getc(FileHandle), i++)
    {
        cpString[i] = cComp;                /* put this character in the array */
    }/* end for */

    cpString[i] = '\0';                     /* null terminate the string */
#if 0
	for (i = 0 ; cpString[i] != '\0'; i++ )
	{
		if (cpString[i] == '~')
			memmove(&cpString[i+1], &cpString[i], strlen(&cpString[i]));
	}
#endif
    if(strcmp(cpString, "END OF FILE") == 0)
    {
        return FALSE;
    }/* end if */
    return TRUE;
} /* end fnEditChunker */

/*FUNCTION:     fnTepEditUnNodeLink
 *PARAMETERS:   TepIDUnNode **ppListRoot    the root you want the nodes attached to
 *              FILE *FileHandle            the file you want to look thru
 *              char cComp                  The Character you want to signify the end of nodes
 *RETURN VALUE: BOOLEAN TRUE                It did what it needed to do
 *              BOOLEAN FALSE               It found an error somewhere, most likely at allocing
 *SIDE EFFECTS: moves the file pointer to the file handle that is passed
 *DESCRIPTION:  This function hacks through and pulls a string from a file, adds it to a linked list of them
 *              it'll stop when it sees the cComp character as the next character.
*/
BOOL fnTepEditUnNodeLink(TepIDUnNode **ppListRoot, FILE *FileHandle, char cComp)
{
    TepIDUnNode     *pCurrNode = *ppListRoot;               /* point to the first item in the list*/
    TepIDUnNode     *pNewNode = NULL;                       /* pointer to the newly created node */
    BOOL            bErrorCheck = TRUE;                     /* varible to hold bool results */
    char            cpString[SIZE_OF_ARRAY];                    /* string filled with file chunks */

    memset(cpString, 0, SIZE_OF_ARRAY );                        /* zero out the string */

    for( ; ; )
    {
        //printf("called\n");
        pNewNode = calloc(1, sizeof(TepIDUnNode));                  /* allocate memory for the new node */

		if (pCurrNode == NULL)
			*ppListRoot = pCurrNode = pNewNode;

        if(pNewNode == NULL)                                        /* check for failure */
        {
            printf("failed to calloc memory for the TepIDUnNode\n");
            return FALSE;
        }

        pNewNode->NEXT = NULL;                                      /* set the next node pointer to NULL */

        if(fnEditChunker(cpString, FileHandle, ',') == FALSE)       /* get the next chunk of info, check for END OF FILE */
        {

            break;
        }/* end if */
        /* fill in the id, check for the use of #defs */

//      printf("Attach this: %s\n", cpString);

        if(atoi(cpString) == 0)                                     /* If this is a #def name, call fnDefToHex */
        {                                                           /*  to convert it to its number. */
  //          pNewNode->NODE = fnDefToHex(cpString);
			fnTepIDAppend(ppListRoot, fnDefToHex(cpString));
			printf("This was attached, 0x%X\n",fnDefToHex(cpString));
        }/* end if */
        else                                                        /* If it is just a number use that */
        {
            pNewNode->NODE = atoi(cpString);
        }/* end else *///0x300BF0

        /* Test the node(s) */
		//printf("%08X or %d\n", pNewNode->NODE, pNewNode->NODE);       /* see what we've got */

        pCurrNode->NEXT = pNewNode;                                 /* set this node to the list */
        pCurrNode = pNewNode;                                       /* point CurrNode at this node */

        if(getc(FileHandle) == cComp)                               /* pull out the next character in the file */
        {
            break;                                                  /* break if it matches the cComp */
        }/* end if */

        /* if not move the file pointer back by one */
        if(fseek(FileHandle, -1, SEEK_CUR ) != 0 )              /* check for failure to reset pointer */
        {
            printf("failed to reset the file pointer in fnTepEditUnNodeLink\n");
            return FALSE;
        }
    }/* end for */

    return TRUE;
}

/*FUNCTION:     fnEditChunker
 *PARAMETERS:   char *cpString              #define name you're looking for
 *RETURN VALUE: TepID                       the #define value of what you sent in or 0 for nothing
 *                                          ie "totok" would return "TEP_CHARACTER | CHAR_TOTOK". etc
 *SIDE EFFECTS: none
 *DESCRIPTION:  Hacks through and uses strcmp to compair names until it finds what it is looking for.
 *              It may be chaunky but it does the job...
*/
TepID fnDefToHex(char *cpString)
{
	TepID IDRET = 0;
    int i = 0;                          /* counter */
	static int b = 0;

    /* convert the string to all upper case */
    for(i = 0 ; cpString[i] != 0 ; i++)
    {
        cpString[i] = toupper(cpString[i]);
    }

		//printf("---%d--%s\n", b++, cpString);

    /* use strcmp to compair the string to all character names in else if statements, */
    /* if there is a match return that character's TepID */
/*************************BEGIN TEP_CHARACTER *************************/
    if(strcmp(cpString, "ALAQUA" ) == 0)
    {
        IDRET =  TEP_CHARACTER |  CHAR_ALAQUA;
    }/* end if */
    else if(strcmp(cpString, "AMALIYA") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_AMALIYA;
        }/* end else if */
    else if(strcmp(cpString, "ANOKI") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_ANOKI;
        }/* end else if */
    else if(strcmp(cpString, "COMAN") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_COMAN;
        }/* end else if */
    else if(strcmp(cpString, "FIALA") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_FIALA;
        }/* end else if */
    else if(strcmp(cpString, "HANIA") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_HANIA;
        }/* end else if */
	else if(strcmp(cpString, "HINUN") == 0)
		{
            IDRET =  TEP_CHARACTER |  CHAR_HINUN;
        }/* end else if */
    else if(strcmp(cpString, "IMALA") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_IMALA;
        }/* end else if */
    else if(strcmp(cpString, "IOAKIM") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_IOAKIM;
        }/* end else if */
    else if(strcmp(cpString, "ISAK") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_ISAK;
        }/* end else if */
    else if(strcmp(cpString, "KAILA") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_KAILA;
        }/* end else if */
    else if(strcmp(cpString, "KALIL") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_KALIL;
        }/* end else if */
    else if(strcmp(cpString, "KAVERI") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_KAVERI;
        }/* end else if */
    else if(strcmp(cpString, "LEILANI") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_LEILANI;
        }/* end else if */
    else if(strcmp(cpString, "MERRIPEN") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_MERRIPEN;
        }/* end else if */
    else if(strcmp(cpString, "QIANRU") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_QIANRU;
        }/* end else if */
    else if(strcmp(cpString, "RUSHI") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_RUSHI;
        }/* end else if */
    else if(strcmp(cpString, "SENWE") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_SENWE;
        }/* end else if */
    else if(strcmp(cpString, "TOTOK") == 0)
        {
            IDRET =  TEP_CHARACTER | CHAR_TOTOK;
        }/* end else if */
    else if(strcmp(cpString, "WINGI") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_WINGI;
        }/* end else if */
    else if(strcmp(cpString, "SHEROESHA") == 0)
        {
            IDRET =  TEP_CHARACTER |  CHAR_SHEROESHA;
        }/* end else if */
	else if(strcmp(cpString, "CHAR_GEN") == 0)
	    {
            IDRET =  TEP_CHARACTER |  CHAR_GEN;
        }/* end else if */
/*************************BEGIN TEP_TALK *************************/
   else if(strcmp(cpString, "ANSEVIDENCEHELLYES") == 0)
        {
            IDRET =  TEP_TALK |  TALK_ANSEVIDENCEHELLYES;
        }/* end else if */
   else if(strcmp(cpString, "ANSEVIDENCEHELLNO") == 0)
        {
            IDRET =  TEP_TALK |  TALK_ANSEVIDENCEHELLNO;
        }/* end else if */
   else if(strcmp(cpString, "EODINTRO") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODINTRO;
        }/* end else if */
   else if(strcmp(cpString, "EODHILLRUN") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODHILLRUN;
        }/* end else if */
   else if(strcmp(cpString, "EODHAPPYSPEECH") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODHAPPYSPEECH;
        }/* end else if */
   else if(strcmp(cpString, "EODSADSPEECH") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODSADSPEECH;
        }/* end else if */
   else if(strcmp(cpString, "BOGINTROSPEECH") == 0)
        {
            IDRET =  TEP_TALK |  TALK_BOGINTROSPEECH;
        }/* end else if */
   else if(strcmp(cpString, "EODACCUSATION") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODACCUSATION;
        }/* end else if */
   else if(strcmp(cpString, "EODCRUMBLE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODCRUMBLE;
        }/* end else if */
   else if(strcmp(cpString, "EODNOCRUMBLE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODNOCRUMBLE;
        }/* end else if */
   else if(strcmp(cpString, "EODCLOSINGSPEECH") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EODCLOSINGSPEECH;
        }/* end else if */
   else if(strcmp(cpString, "SACRIFICE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_SACRIFICE;
        }/* end else if */
   else if(strcmp(cpString, "EOGWINS") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EOGWINS;
        }/* end else if */
   else if(strcmp(cpString, "EOGLOSE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EOGLOSE;
        }/* end else if */
   else if(strcmp(cpString, "CLUERECALL") == 0)
        {
            IDRET =  TEP_TALK |  TALK_CLUERECALL;
        }/* end else if */
   else if(strcmp(cpString, "INSULTREACTION") == 0)
        {
            IDRET =  TEP_TALK |  TALK_INSULTREACTION;
        }/* end else if */
   else if(strcmp(cpString, "POSITIVECOMPLIMENT") == 0)
        {
            IDRET =  TEP_TALK |  TALK_POSITIVECOMPLIMENT;
        }/* end else if */
   else if(strcmp(cpString, "NEGATIVECOMPLIMENT") == 0)
        {
            IDRET =  TEP_TALK |  TALK_NEGATIVECOMPLIMENT;
        }/* end else if */
   else if(strcmp(cpString, "BOGSHEROESHASPEAKS") == 0)
        {
            IDRET =  TEP_TALK |  TALK_BOGSHEROESHASPEAKS;
        }/* end else if */
   else if(strcmp(cpString, "TALKABOUTDEAD") == 0)
        {
            IDRET =  TEP_TALK |  TALK_TALKABOUTDEAD;
        }/* end else if */
   else if(strcmp(cpString, "BOGINTRODUCTION") == 0)
        {
            IDRET =  TEP_TALK |  TALK_BOGINTRODUCTION;
        }/* end else if */
   else if(strcmp(cpString, "BODWAKEUP") == 0)
        {
            IDRET =  TEP_TALK |  TALK_BODWAKEUP;
        }/* end else if */
   else if(strcmp(cpString, "BODWAKEUP") == 0)
        {
            IDRET =  TEP_TALK |  TALK_BODWAKEUP;
        }/* end else if */
   else if(strcmp(cpString, "EOGWININTRO") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EOGWININTRO;
        }/* end else if */
   else if(strcmp(cpString, "EOGWINOUTRO") == 0)
        {
            IDRET =  TEP_TALK |  TALK_EOGWINOUTRO;
        }/* end else if */
	else if(strcmp(cpString, "GENINSULT") == 0)
        {
            IDRET =  TEP_TALK |  TALK_GENINSULT;
        }/* end else if */
    else if(strcmp(cpString, "GENCOMPLIAMENT") == 0)
        {
            IDRET =  TEP_TALK |  TALK_GENCOMPLIAMENT;
        }/* end else if */
    else if(strcmp(cpString, "HELLO") == 0)
        {
            IDRET =  TEP_TALK |  TALK_HELLO;
        }/* end else if */
    else if(strcmp(cpString, "ANSQUERY") == 0)
        {
            IDRET =  TEP_TALK |  TALK_ANSQUERY;
        }/* end else if */
    else if(strcmp(cpString, "ANSEVIDENCEYES") == 0)
        {
            IDRET =  TEP_TALK |  TALK_ANSEVIDENCEYES;
        }/* end else if */
    else if(strcmp(cpString, "ANSEVIDENCENO") == 0)
        {
            IDRET =  TEP_TALK |  TALK_ANSEVIDENCENO;
        }/* end else if */
    else if(strcmp(cpString, "CLUESLOCATION") == 0)
        {
            IDRET =  TEP_TALK |  TALK_CLUESLOCATION;
        }/* end else if */
    else if(strcmp(cpString, "CLUESCHARACTER") == 0)
        {
            IDRET =  TEP_TALK |  TALK_CLUESCHARACTER;
        }/* end else if */
    else if(strcmp(cpString, "INIT") == 0)
        {
            IDRET =  TEP_TALK |  TALK_INIT;
        }/* end else if */
    else if(strcmp(cpString, "GENFAILURE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_GENFAILURE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_PERSONALCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_PERSONALCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_ABANDONEDCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_ABANDONEDCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_SANDYCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_SANDYCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_FOLIAGECLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_FOLIAGECLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_RAVINECLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_RAVINECLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_CORALCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_CORALCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_WORKCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_WORKCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_COMMUNALCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_COMMUNALCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_WATERCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_WATERCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_WARMTHCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_WARMTHCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_GRAVECLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_GRAVECLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_SWAMPCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_SWAMPCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_CHILDCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_CHILDCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_SWAMPSANDCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_SWAMPSANDCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_DARKCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_DARKCLUE;
        }/* end else if */
	else if(strcmp(cpString, "TALK_PLAINSCLUE") == 0)
        {
            IDRET =  TEP_TALK |  TALK_PLAINSCLUE;
        }/* end else if */
/*************************BEGIN TEP_COMMAND *************************/
    else if(strcmp(cpString, "GO") == 0)
        {
            IDRET =  COM_GO;
        }/* end else if */
    else if(strcmp(cpString, "MOVE") == 0)
        {
            IDRET =  COM_GO;
        }/* end else if */
    else if(strcmp(cpString, "WALK") == 0)
        {
            IDRET =  COM_GO;
        }/* end else if */
    else if(strcmp(cpString, "LEAVE") == 0)
        {
            IDRET =  COM_LEAVE;
        }/* end else if */
    else if(strcmp(cpString, "NORTH") == 0)
        {
            IDRET =  COM_NORTH;
        }/* end else if */
    else if(strcmp(cpString, "SOUTH") == 0)
        {
            IDRET =  COM_SOUTH;
        }/* end else if */
    else if(strcmp(cpString, "EAST") == 0)
        {
            IDRET =  COM_EAST;
        }/* end else if */
    else if(strcmp(cpString, "WEST") == 0)
        {
            IDRET =  COM_WEST;
        }/* end else if */
    else if(strcmp(cpString, "PUT") == 0)
        {
            IDRET =  COM_PUT;
        }/* end else if */
    else if(strcmp(cpString, "GRAB") == 0)
        {
            IDRET =  COM_PUT;
        }/* end else if */
    else if(strcmp(cpString, "TAKE") == 0)
        {
            IDRET = COM_PUT;
        }/* end else if */
    else if(strcmp(cpString, "SWIPE") == 0)
        {
            IDRET =  COM_PUT;
        }/* end else if */
    else if(strcmp(cpString, "LOOK") == 0)
        {
            IDRET =  COM_LOOK;
        }/* end else if */
    else if(strcmp(cpString, "EXAMINE") == 0)
        {
            IDRET =  COM_LOOK;
        }/* end else if */
    else if(strcmp(cpString, "SEE") == 0)
        {
            IDRET =  COM_LOOK;
        }/* end else if */
    else if(strcmp(cpString, "INSPECT") == 0)
        {
            IDRET =  COM_LOOK;
        }/* end else if */
    else if(strcmp(cpString, "CHECK") == 0)
        {
            IDRET =  COM_LOOK;
        }/* end else if */
    else if(strcmp(cpString, "VIEW") == 0)
        {
            IDRET =  COM_VIEW;
        }/* end else if */
    else if(strcmp(cpString, "HELP") == 0)
        {
            IDRET =  COM_HELP;
        }/* end else if */
    else if(strcmp(cpString, "SAVE") == 0)
        {
            IDRET =  COM_SAVE;
        }/* end else if */
    else if(strcmp(cpString, "LOAD") == 0)
        {
            IDRET =  COM_LOAD;
        }/* end else if */
    else if(strcmp(cpString, "QUIT") == 0)
        {
            IDRET =  COM_QUIT;
        }/* end else if */
    else if(strcmp(cpString, "GREET") == 0)
        {
            IDRET =  COM_GREET;
        }/* end else if */
    else if(strcmp(cpString, "HELLO") == 0)
        {
            IDRET =  COM_GREET;
        }/* end else if */
    else if(strcmp(cpString, "HI") == 0)
        {
            IDRET =  COM_GREET;
        }/* end else if */
    else if(strcmp(cpString, "INSULT") == 0)
        {
            IDRET =  COM_INSULT;
        }/* end else if */
    else if(strcmp(cpString, "TALK") == 0)
        {
            IDRET =  COM_TALK;
        }/* end else if */
    else if(strcmp(cpString, "SPEAK") == 0)
        {
            IDRET =  COM_TALK;
        }/* end else if */
    else if(strcmp(cpString, "ASK") == 0)
        {
            IDRET =  COM_ASK;
        }/* end else if */
    else if(strcmp(cpString, "SHOW") == 0)
        {
            IDRET =  COM_SHOW;
        }/* end else if */
    else if(strcmp(cpString, "GOODBYE") == 0)
        {
            IDRET =  COM_GOODBYE;
        }/* end else if */
    else if(strcmp(cpString, "CHECK") == 0)
        {
            IDRET =  COM_LOOK;
        }/* end else if */
    else if(strcmp(cpString, "DONE") == 0)
        {
            IDRET =  COM_DONE;
        }/* end else if */
    else if(strcmp(cpString, "REST") == 0)
        {
            IDRET =  COM_DONE;
        }/* end else if */
    else if(strcmp(cpString, "WAIT") == 0)
        {
            IDRET =  COM_DONE;
        }/* end else if */
	else if(strcmp(cpString, "COMPLIMENT") == 0)
        {
            IDRET =  COM_COMPLIMENT;
        }/* end else if */
/*************************BEGIN TEP_CLUE *************************/
	else if(strcmp(cpString, "LOCATION") == 0)
        {
            IDRET =  TEP_CLUE |  CLUE_LOCATION;
        }/* end else if */
    else if(strcmp(cpString, "PERPITEM") == 0)
        {
            IDRET =  TEP_CLUE | CLUE_PERPITEM;
        }/* end else if */
    else if(strcmp(cpString, "PERPLOCA") == 0)
        {
            IDRET =  TEP_CLUE | CLUE_PERPLOCA;
        }/* end else if */
    else if(strcmp(cpString, "PERPVERBAL") == 0)
        {
            IDRET =  TEP_CLUE | CLUE_PERPVERBAL;
        }/* end else if */
    else if(strcmp(cpString, "HERRING") == 0)
        {
            IDRET =  TEP_CLUE | CLUE_PERPVERBAL;
        }/* end else if */
    else if(strcmp(cpString, "FROMDEATHONPERP") == 0)
        {
            IDRET =  TEP_CLUE | CLUE_FROMDEATHONPERP;
        }/* end else if */
    else if(strcmp(cpString, "FROMDEATHATLOCA") == 0)
        {
            IDRET =  TEP_CLUE | CLUE_FROMDEATHATLOCA;
        }/* end else if */
//*************************BEGIN TEP_ROOM *************************/
	else if(strcmp(cpString, "ROOM_BEACH1") == 0)
		{
			IDRET = TEP_ROOM |  ROOM_ROOM | ROOM_BEACH1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_BEACH2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_BEACH2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_BEACH3") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_BEACH3;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_BEANPATCH") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_BEANPATCH;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_BOFWATER") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_BOFWATER;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_BRIDGE") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_BRIDGE;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_CLIFFS1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_CLIFFS1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_CLIFFS2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_CLIFFS2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_COOKHUT") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_COOKHUT;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_CORALREEF1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_CORALREEF1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_CORALREEF2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_CORALREEF2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_DESPATH") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_DESPATH;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_EATHUT") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_EATHUT;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_FIELD") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_FIELD;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_FORPATH1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_FORPATH1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_FORPATH2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_FORPATH2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_FORPATH3") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_FORPATH3;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_GEYSER") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_GEYSER;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_GRAVE1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_GRAVE1;
		}
	else if(strcmp(cpString, "ROOM_GRAVE2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_GRAVE2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_GROGROCK") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_GROGROCK;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_HOTSPRINGS") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HOTSPRINGS;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_HUT1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HUT1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_HUT2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HUT2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_HUT3") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HUT3;
		}
	else if(strcmp(cpString, "ROOM_HUT4") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HUT4;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_HUT5") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HUT5;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_HUT6") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HUT6;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_HUT7") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_HUT7;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_LAKESIDE1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_LAKESIDE1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_LAKESIDE2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_LAKESIDE2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_LONGHOUSEA") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_LONGHOUSEA;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_LONGHOUSEB") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_LONGHOUSEB;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_LONGHOUSEC") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_LONGHOUSEC;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_LONGHOUSED") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_LONGHOUSED;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_KIDDIELONG") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_KIDDIELONG;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_KIDDIEPLAY") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_KIDDIEPLAY;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_MARSH1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_MARSH1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_MARSH2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_MARSH2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_MARSH3") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_MARSH3;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_MARSH4") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_MARSH4;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_MARSH5") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_MARSH5;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_MARSH6") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_MARSH6;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_PROVINCE") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_PROVINCE;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_SEACAVE") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_SEACAVE;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_SWAMPYBEACH") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_SWAMPYBEACH;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_TEMPLERAM") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_TEMPLERAM;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_TEMPLETEP") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_TEMPLETEP;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_VILLAGE1") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_VILLAGE1;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_VILLAGE2") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_VILLAGE2;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_VILLAGE3") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_VILLAGE3;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_VILLAGE4") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_VILLAGE4;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_VOLCANOPATH") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_VOLCANOPATH;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_WEAVEHUT") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_WEAVEHUT;
		}/* end else if */
	else if(strcmp(cpString, "ROOM_WORKSHOP") == 0)
		{
			IDRET =   TEP_ROOM |  ROOM_ROOM | ROOM_WORKSHOP;
		}/* end else if */
    if (!(IDRET & (TEP_CLUE | TEP_COMMAND | TEP_TALK | TEP_ROOM | TEP_CHARACTER | TEP_DEATH)))
    {
		if (cpString[0] != '0')
		{
			//printf("---%d--", b++);
			//printf("\'%s\' = %08X\n", cpString, IDRET);
			return 0;
		}
	}

//	printf("IDRET is 0x%X\n", IDRET);
    return IDRET;
}/* end function fnDefToHex */


/*FUNCTION:     fnTepEditWriteFile
 *PARAMETERS:   DWORD dwType                #defined type of file you are writing
 *              void *pListRoot             pointer to the ListRoot
 *              FILE *outFile               this is the file handle used for the output of the generated file. Pass NULL to print to screen.
 *RETURN VALUE: BOOL                        SUCCESS/FAILURE
 *SIDE EFFECTS: data file generated
 *DESCRIPTION:  Based on the type given, the function reads the entire dataset and prints it to file.
*/
BOOL fnTepEditWriteFile(const DWORD dwType, void *pListRoot, FILE *outFile)
{
    BOOL fRet = FAILURE;

    if (pListRoot != NULL)
    {
        void *pCurNode = pListRoot;
        outFile = (outFile) ? (outFile) : (stdout);
        while (pCurNode != NULL)
        {
            static DWORD dwWriteZero = 0;
            switch (dwType)
            {
                case TEP_CLUE:
                {
                    void *lpvTmp = NULL;

                    /* write the id to the file */
                    fwrite(&((pTepClueData)pCurNode)->id, sizeof(((pTepClueData)pCurNode)->id), 1, outFile);

                    /* write the attached to list to the file */
                    fwrite(&((pTepClueData)pCurNode)->attachedto, sizeof(((pTepClueData)pCurNode)->attachedto), 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepClueData)pCurNode)->details,0);
                    //fwrite(&((pTepClueData)pCurNode)->details, sizeof(((pTepClueData)pCurNode)->details[0]), strlen(((pTepClueData)pCurNode)->details) + 1, outFile);

                    pCurNode = ((pTepClueData)pCurNode)->NEXT;
                    break;
                }

                case TEP_ROOM:
                {
                    printf("Room handling functionality not implemented.\n");

                    pCurNode = NULL;
                    break;
                }

                case TEP_DEATH:
                {
                    /* write the id to the file */
                    fwrite(&((pTepDeath)pCurNode)->id, sizeof(((pTepDeath)pCurNode)->id), 1, outFile);

                    /* write the location to the file */
                    fwrite(&((pTepDeath)pCurNode)->location, sizeof(((pTepDeath)pCurNode)->location), 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepDeath)pCurNode)->desc,0);
                    //fwrite(&((pTepDeath)pCurNode)->desc, sizeof(((pTepDeath)pCurNode)->desc[0]), strlen(((pTepDeath)pCurNode)->desc) + 1, outFile);

                    pCurNode = ((pTepDeath)pCurNode)->NEXT;
                    break;
                }

                case TEP_TALK:
                {
                    /* write the id to the file */
                    fwrite(&((pTepDBConvData)pCurNode)->id, sizeof(((pTepDBConvData)pCurNode)->id), 1, outFile);

                    /* write the attachment to the file */
                    fwrite(&((pTepDBConvData)pCurNode)->attachedto, sizeof(((pTepDBConvData)pCurNode)->attachedto), 1, outFile);

                    /* write the owner to the file */
                    fwrite(&((pTepDBConvData)pCurNode)->owner, sizeof(((pTepDBConvData)pCurNode)->owner), 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepDBConvData)pCurNode)->details,0);
                    //fwrite(&((pTepDBConvData)pCurNode)->details, sizeof(((pTepDBConvData)pCurNode)->details[0]), strlen(((pTepConvData)pCurNode)->details) + 1, outFile);

                    pCurNode = ((pTepConvData)pCurNode)->NEXT;
                    break;
                }

                case TEP_CHARACTER:
                {
                    void *lpvTmp = NULL;

                    /* write the id to the file */
                    fwrite(&((pTepChar)pCurNode)->id, sizeof(((pTepChar)pCurNode)->id), 1, outFile);
                    printf("\n[%08X]\n", ((pTepChar)pCurNode)->id);

                    /* write the null-terming string with the name */
                    fprintf(outFile, "%s%c", ((pTepChar)pCurNode)->name,0);
                    printf("%s%c\n", ((pTepChar)pCurNode)->name,0);
                    //fwrite(((pTepChar)pCurNode)->name, sizeof(((pTepChar)pCurNode)->name[0]), strlen(((pTepChar)pCurNode)->name) + 1, outFile);

                    /* write the null-terming string with the details */
                    fprintf(outFile, "%s%c", ((pTepChar)pCurNode)->desc,0);
                    printf("%s%c\n", ((pTepChar)pCurNode)->desc,0);
                    //fwrite(&((pTepChar)pCurNode)->desc, sizeof(((pTepChar)pCurNode)->desc[0]), strlen(((pTepChar)pCurNode)->desc) + 1, outFile);

                    /* write the attached to list to the file */
                    lpvTmp = ((pTepChar)pCurNode)->locations;
                    while (lpvTmp != NULL)
                    {
						if (((pTepIDUnNode)lpvTmp)->NODE != 0)
						{
                        	fwrite(&((pTepIDUnNode)lpvTmp)->NODE, sizeof(((pTepIDUnNode)lpvTmp)->NODE), 1, outFile);
                        	printf("%08X={%08X,%08X}\n", lpvTmp, ((pTepIDUnNode)lpvTmp)->NODE, ((pTepIDUnNode)lpvTmp)->NEXT);
						}
                        lpvTmp = ((pTepIDUnNode)lpvTmp)->NEXT;
                    }
                    /* write the marker for end of attachment list */
                    fwrite(&dwWriteZero, sizeof(dwWriteZero), 1, outFile);
                    printf("%08X//\n",0);

                    pCurNode = ((pTepChar)pCurNode)->NEXT;
                    break;
                }

                default:
                    break;
            }
        }
    }

    return fRet;
}
