/**************************************************************
*** VERSION NUMBER 0002
*
***************************************************************
* Author :
*   Jon Olson <jolson@digipen.edu>
*
* Name :        tep_incs.h
* Header :      none
*
* Objective:
*   This has been written as an include for all source code
*   files of Tep to include.  This should be the only thing
*   needed to include.
*
* Methods :
*   Uses modal information with other values and globals to
*   decide how to retrieve the information requested
*
* Archives necessary : none
*
* Environments :
*   this will not load anything if a windows file is being
*   created.
*
* Requirements of the Hardware : none
*
* Restrictions : console only
*
* Mode of Use :
*   the program uses preprocessor functionality to load
*   the particular headers and data needed to compile
*   as long as this is not being used in an outside of
*   console application.
*
* Compiler : vc6
*
* Command to compile : none, its a header!
*
* Bibliography :
*   Jon Olson (wetware)
*
*/

#pragma once


#if !defined(WINVER)

    /* standard i/o functionality */
    #include <stdio.h>

    /* variable arguments functionality */
    #if !defined(UNIX)

        #include <stdarg.h>

    #else

        #include <vararg.h>

    #endif

    /*
        string manipulation functionality,
        this should be removed when a string manipulate module can
        be written to handle the functionality of this.
    */
    #include <string.h>

    /* standard memory allocation and destruction functionality */
    #include <malloc.h>

    /* standard time functionality (randomizers) */
    #include <time.h>

    /* all resources used 'The Emissary of Tep' */
    //#include "tep_resource.h"

    /* all datatypes in use by 'The Emissary of Tep' */
    #include "tep_types.h"


/***** BEGIN MATT'S ADDITIONS ************************/

    /* constants */
    #include "tep_constants.h"

    /* standard library stuff... */
    #include <conio.h>
    #include <stdlib.h>

    /* tep_main.c functions */
    #include "tep_main.h"

	/* tep_fileio.c functions */
	#include "tep_fileio.h"

    /* tep_lists.c functions */
    #include "tep_lists.h"

    /* tep_printout.c functions */
    #include "tep_printout.h"

    /* tep_parse.c functions */
    #include "tep_parse.h"

    /* tep_int_act.c functions */
    #include "tep_int_act.h"

    /* tep_quit.c functions */
    #include "tep_quit.h"

    /* tep_examine.c functions */
    #include "tep_examine.h"

    /* tep_show.c functions */
    #include "tep_show.h"

    /* tep_ev_collect.c functions */
    #include "tep_ev_collect.h"

    /* tep_greet.c functions */
    #include "tep_greet.h"

    /* tep_insult.c functions */
    #include "tep_insult.h"

    /* tep_sacrifice.c functions */
    #include "tep_sacrifice.h"

    /* tep_gameover.c functions */
    #include "tep_gameover.h"

    /* tep_newturn.c functions */
    #include "tep_newturn.h"

    /* tep_endturn.c functions */
    #include "tep_endturn.h"

    /* tep_proto_data.c functions */
    #include "tep_proto_data.h"

	/* tep_story_intro.c functions */
	#include "tep_story_intro.h"

	/* STUFF */
	#include "RoomDefines.h"

	/* character constants */
	#include "CharacterDefines.h"

	/* character picking functions */
	#include "tep_charpick.h"

	#include "tep_ev_organize.h"

	#include "tep_compliment.h"

	#include "ActionCost.h"

	#include "tep_int_talk.h"

	#include "RoomEdit.h"

	#include "tep_move.h"

	#include "TepEditUtils.h"

	#include "tepkeywordgenerator.h"

	#include "TepEditCharacter.h"

	#include "TepEditClue.h"

	#include "TepEditCommand.h"

	#include "TepEditTalk.h"

	#include "TepEditDeath.h"

	#include "AttachTalk.h"

	#include "tep_ask.h"

	#include "tep_saveload.h"

	#include "tep_sherosclue.h"

#include "tep_HelpFile.h"



/***** END MATT'S ADDITIONS ************************/


   /* #if defined(_DEBUG)

         add debugging structures and data
        #warning Adding debugging functionality...
        #include "tep_debugging.h"

    #elif !defined(_RETAIL)

        #error _DEBUG or _RETAIL must be defined!

    #endif */

#else

    #error Compile for console applications only

#endif



/* EOF */
