/*PROGRAM ID:           EvidenceCollect.c
 *PROGRAMMER NAME:      John Corpening and Matt Phillips
 *DATE CREATED:         February 6, 2001
 *PROGRAM UPDATED BY:                       Date of Update:
 *PROGRAM DESCRIPTION:  Program to collect Evidence
 *DEPENDENCIES:         "tep_incs.h"
 *INPUT FILES:          none
 *OUTPUT FILES:         none
*/

#include "tep_incs.h"


/*FUNCTION:     fnEvidenceCollect
 *PARAMETERS:   TepCurState             State       Takes this to get to the player info
 *              TepParse                msg         Information sent from the parser as to what to append
 *RETURN VALUE: 1 for Success or 0 for failure.
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function is used to collect evidence
*/
BOOL fnEvidenceCollect( TepCurState *State, TepParse msg )
{
    TepDynRoom  *pRoom;     /*Pointer to a dynamic room needed for checking if the clue is ther*/
	TepID		CharID = PARSE_UNUSED;

    /*Check to make sure that evidence is in the Current room*/
    pRoom = fnTepDynRoomSearch( State->mapLite, State->currentRoom->id );

	if( pRoom != NULL )
	{
		if( fnTepIDSearch( pRoom->objs, msg.Clue ) == NULL )			
		{
			if( !fnEvidenceCheckChars( State->liveList, pRoom->people, msg.Clue, &CharID ) )
			{
				printf( "You cannot do that.\n" );
				return FAILURE;
			}
		}

		/* if no character was specified give the command the id of the generic or null character */
		if( msg.Char1 == PARSE_UNUSED )
		{
			msg.Char1 = TEP_CHARACTER | CHAR_GEN;
		}

		/*Check to see if player has already collected evidence*/
		if( fnTepEvidenceSearch( State->player->evidenceGeneric, msg.Clue, 0 ) != NULL )
		{
			printf("This is already recorded in your journal.\n");
			return FAILURE;
		}
		else
		{
			/* if not -  append it to the evidence list */
			if( CharID != PARSE_UNUSED )
			{
				fnTepEvidenceAppend( &State->player->evidenceGeneric, msg.Clue, msg.Char1, CharID );
			}
			else
			{
				fnTepEvidenceAppend( &State->player->evidenceGeneric, msg.Clue, msg.Char1, pRoom->asRoomID );
			}
			printf( "The evidence has been recorded in your journal.\n" );
			return SUCCESS;
		}
	}

	/* there isn't any evidence here */
	printf("There isn't any evidence here.\n");

    return FAILURE;
}

/*FUNCTION:     fnEvidencePrint
 *PARAMETERS:   TepPlayerEvidenceBiNode *pNode      Used as the starting point for the list
 *RETURN VALUE: 1 for Success.
 *SIDE EFFECTS: Prints to the screen the contents of each node
 *DESCRIPTION:  This functiond prints everything in an evidence list.
*/
int fnEvidencePrint(TepPlayerEvidence *pNode)
{
    TepPlayerEvidence *pCurrent; /*Pointer to the current node*/

    for(pCurrent = pNode; pCurrent != NULL; pCurrent = pCurrent->NEXT)
    {
        printf("Clue: %d\tCharacter: %d\n", pCurrent->clue, pCurrent->attachedTo);

    }
    return 1;
}



BOOL fnEvidenceCheckChars( TepChar *pLiveList, TepIDUnNode *pPeepList, TepID EvID, TepID *pCharID )
{
	TepChar *pPeepData;
	TepIDUnNode *pPeepsClue;

	/* loop through the dynamic room's characters */
	for( ; pPeepList != NULL; pPeepList = pPeepList->NEXT )
	{
		if( ( pPeepData = fnTepCharSearch( pLiveList, pPeepList->NODE ) ) != NULL )
		{
			if( ( pPeepsClue = fnTepIDSearch( pPeepData->clues, EvID ) ) != NULL )
			{
				*pCharID = pPeepData->id;
				return TRUE;
			}
		}
	}

	return FALSE;
}


/* Test Stuff
int main(void)
{
    TepPlayer   player;
    TepParse    msg[10];
    int i;

    player.evidenceAgainst = NULL;
    player.evidenceGeneric = NULL;
    player.name = "bob";
    player.oldEvidenceAgainst = NULL;
    player.oldEvidenceGeneric = NULL;

    for(i = 0; i < 10; i++)
    {
        msg[i].Char1 = i%3;
        msg[i].Clue = i;
        fnEvidenceCollect(&player, msg[i]);
    }

    printf("Evidence Against\n");
    fnEvidencePrint(player.evidenceAgainst);
    printf("Evidence Generic\n");
    fnEvidencePrint(player.evidenceGeneric);
    return 0;
}

*/
