#include "tep_incs.h"


/*FUNCTION:     fnEvidenceOrganize
 *PARAMETERS:   TepCurState             *pGameState     Takes this to get to the player info
 *              TepParse                msg             Information sent from the parser as to what to append
 *RETURN VALUE: 1 for Success or 0 for failure.
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function is used to view and organize evidence
*/
BOOL fnEvidenceOrganize( TepCurState *pGameState, TepParse *msg )
{
    TepPlayerEvidence *pCurEvList = pGameState->player->evidenceGeneric;
    TepClueData *pCurClueList = NULL;
    TepCurMurder *pCurMurder = NULL;
    char sBuff[INPUT_SIZE];



    /* find the current evidence list and the current clue list */
    /* just get the current murder */
    pCurMurder = pGameState->currentMurder;
    if( pCurMurder == NULL )
    {
        return FAILURE;
    }
    else
    {
        while( pCurMurder->NEXT != NULL )
        {
            pCurMurder = pCurMurder->NEXT;
        }
    }

    /* now we can get the current clue list */
    pCurClueList = pCurMurder->death->clues;

	/* display the initial list */
	fnEvOrgDisplay( pGameState, pCurClueList, pCurEvList, msg );

    /* loop until user chooses to leave */
    for( ; ; )
    {
        /* interpret msg */
        if( !fnEvOrgInterpret( pGameState, pCurClueList, pCurEvList, msg ) )
        {
            return SUCCESS;
        }

        /* display a prompt */
        fnIntActPrompt( pGameState, "Evidence" );

        /* get user input */
        gets( sBuff );

        /* parse input */
        fnParseText( sBuff, msg, pGameState );
    }

    return SUCCESS;
}


BOOL fnEvOrgInterpret( TepCurState *pGameState, TepClueData *pClueList, TepPlayerEvidence *pEvList, TepParse *msg )
{

	//printf("Command: %08x", msg->Command);

    /* check the command recieved */
    switch( msg->Command )
    {
        /* the user wants to exit the organization system */
        case COM_GO:
			if( msg->Reserved == COM_NORTH )
	            return FALSE; /* FALSE will be interpreted as: leave the evidence organization */
			else
				printf("Invalid Command\n");

        /* the user wants to move some evidence around */
        case COM_PUT:
            fnEvOrgMove( pGameState, pEvList, msg );
            break;

        /* the user wants to display some evidence - a list or an item */
		case COM_LOOK:
        case COM_VIEW:
			/* set the generic evidence character if appropriate */
			if( msg->Char1 == PARSE_UNUSED && msg->Clue == PARSE_UNUSED )
			{
				msg->Char1 = TEP_CHARACTER | CHAR_GEN;
			}

            fnEvOrgDisplay( pGameState, pClueList, pEvList, msg );
            break;

        default:
            /* invalid command */
            printf("Invalid Command\n");
            break;
    }

    return TRUE;
}




BOOL fnEvOrgMove( TepCurState *pGameState, TepPlayerEvidence *pEvList, TepParse *msg )
{
    TepPlayerEvidence *pThisEvidence = NULL;

    /* check that the clue is a valid clue */
    if( ( pThisEvidence = fnTepEvidenceSearch( pEvList, msg->Clue, 0 ) ) == NULL )
    {
        printf("You do not have that clue.\n");
        return FAILURE;
    }

    /* if no character was specified give the command the id of the generic or null character */
    if(msg->Char1 == PARSE_UNUSED)
    {
        msg->Char1 = TEP_CHARACTER | CHAR_GEN;
    }

    /* we have the clue so now change it */
    pThisEvidence->attachedTo = msg->Char1;

    printf("The clue has been moved.\n");

    return SUCCESS;
}




BOOL fnEvOrgDisplay( TepCurState *pGameState, TepClueData *pClueList, TepPlayerEvidence *pEvList, TepParse *msg )
{
    /* if the user wants to display a list... */
    if( msg->Char1 != PARSE_UNUSED )
    {
        /* pass the list id to listDisplay */
        return fnEvOrgListDisplay( pGameState, pClueList, pEvList, msg->Char1 );
    }
    /* if the user wants details on a specific item... */
    else if( msg->Clue != PARSE_UNUSED )
    {
        /* pass the clue id to detailDisplay */
        return fnEvOrgDetailDisplay( pGameState, pClueList, pEvList, msg->Clue );
    }
    else
    {
        /* invalid command */
        return FAILURE;
    }

    return SUCCESS;
}




BOOL fnEvOrgListDisplay( TepCurState *pGameState, TepClueData *pClueList, TepPlayerEvidence *pEvList, TepID Category )
{
    TepPlayerEvidence *pThisEvidence = pEvList;
    TepClueData *pThisClue = NULL;
    TepDataString *pKeyword = NULL;
	unsigned short ItemCount = 0;

    /* get the category name or fail */
	if( Category == ( TEP_CHARACTER | CHAR_GEN ) )
	{
		/* print the category name */
		fnPrintOut( NOPAUSE, NULL,NULL,1, " #GENERAL\n" );
	}
    else if( ( pKeyword = fnTepDataStringSearch( pGameState->keywords, Category ) ) != NULL )
    {
		/* print the category name */
		fnPrintOut( NOPAUSE, NULL,NULL,3, " #", pKeyword->data, "\n" );
    }
	else
	{
		return FAILURE;
	}



    /* now loop through the available evidence and print out any in this category */
    while( pThisEvidence != NULL )
    {
        /* is the item in the category we need? */
        if( pThisEvidence->attachedTo == Category )
        {
            /* get the clue data so we can print it */
            if( ( pThisClue = fnClueDataSearch( pClueList, pThisEvidence->clue, 0 ) ) == NULL )
            {
				#ifdef _DEBUG
				printf("fnEvOrgListDisplay failed to find a clue's data.\n");
				#endif
                return FAILURE;
            }

            /* print the clue keyword */
            fnPrintOut( NOPAUSE, NULL,NULL,2, " #", pThisClue->keyword );

			/* increment the counter */
			ItemCount++;
        }

		/* move to the next clue */
		pThisEvidence = pThisEvidence->NEXT;
    }

	/* check if any items were listed... */
	if( ItemCount == 0 )
	{
		fnPrintOut( NOPAUSE, NULL,NULL,1, " #No evidence in this category");
	}

	printf("\n");

    return SUCCESS;
}




BOOL fnEvOrgDetailDisplay( TepCurState *pGameState, TepClueData *pClueList, TepPlayerEvidence *pEvList, TepID Item )
{
    TepPlayerEvidence *pThisEvidence;
    TepClueData *pThisClue;

    /* find the item in the evidence list or fail */
    if( ( pThisEvidence = fnTepEvidenceSearch( pEvList, Item, 0 ) ) == NULL )
    {
        return FAILURE;
    }

    /* find the clue data so that we can print the data or fail */
    if( ( pThisClue = fnClueDataSearch( pClueList, pThisEvidence->clue, 0 ) ) == NULL )
    {
        return FAILURE;
    }

	/* now display the clue with the proper "where" data */
	if( pThisEvidence->whereFound & TEP_CHARACTER )
	{
		TepChar *pMoron = NULL;

		/* get the character so we have their name */
		if( ( pMoron = fnTepCharSearch( pGameState->liveList, pThisEvidence->whereFound ) ) == NULL )
		{
			return FAILURE;
		}

		/* is it a verbal clue or a character item? */
		if( pThisEvidence->clue & CLUE_PERPVERBAL )
		{
			/* now print out the details.  */
			fnPrintOut( NOPAUSE, NULL, pMoron->name, 4, " #", pThisClue->keyword, " #@ said: ", pThisClue->details );
		}
		else
		{
			/* now print out the details.  */
			fnPrintOut( NOPAUSE | CharGenderFind( pThisEvidence->whereFound ), NULL, pMoron->name, 4, " #", pThisClue->keyword, " #", pThisClue->details );
		}
	}
	else
	{
		TepRoom *pRoom = NULL;

		/* get the room where the clue was found */
		if( ( pRoom = fnTepRoomSearch( pGameState->FullMap, pThisEvidence->whereFound ) ) == NULL )
		{
			return FAILURE;
		}

		/* now print out the clue name and details.  */
		fnPrintOut( NOPAUSE, NULL, NULL, 6, " #", pThisClue->keyword, " - found at ", pRoom->name, " #", pThisClue->details );
	}

    return SUCCESS;
}
