#include "tep_incs.h"

/* Function Name: InterpretAction
 * Programmer Name: Dan Davis-Boxleitner
 * Date Created: February 9th, 2001
 * Program Updated by:
 *
 * Program Description:     This program outputs the current game time
 *                      and takes input from the user. The user's input
 *                      string is passed to fnParseText(), which fills a
 *                      TepParse struct with information about the command.
 *                      This information is then compared to a list of all
 *                      game commands, and the proper command is called.
 *
 * Program dependencies:    "tep_types.h","InterpretAction.h".fnParseText(),
 *                          all game command functions.
 *
 * Takes:                   A pointer to the game state structure
 *
 * Gives:                   A boolean value,
 *                          TRUE means a command was successfully entered
 *                          FALSE means the command failed
 *
 */
BOOL    fnInterpretAction(pTepCurState pGameState)
{
    char    cUserInput[INPUT_SIZE],    /* The user's input */
            cChoice     = 0;    /* User's choice to quit or not */
    TepParse    *pParse,     /* Pointer to the tep parse structure */
                Results;        /* The tep parse structure filled by fnParseText() */

    //printf( "iTime = %d\n", iTime );

    /* As long as it's before 8 pm loop the input */
    while( pGameState->turnsLeft < 2000 )
    {
        /* Initialize the user input to nothing */
        cUserInput[0] = 0;

		/* display the prompt and the current time */
		fnIntActPrompt( pGameState, NULL );

        /* Grab the user's input */
        gets( cUserInput );

        /* Print out the user's input */
        //printf( "\n'%s'\n", cUserInput );

        pParse = &Results;

        /* Call the parse function. This takes the input and fills the pParse array */
        fnParseText( cUserInput, pParse, pGameState );

		#if defined(_DEBUG)
		printf( "Command: %08x\n", pParse->Command );
		#endif

        /* Switch statement for commands to be interpreted */
        switch ( pParse->Command )
        {
            case COM_GO:
                if( fnDisplayPlayerLocation( pGameState, *pParse ) )
				{
					/* time is now incremented in fnDisplayPlayerLocation, added by nate 3/26/01 */
	                pGameState->turnsLeft += ACT_GO;
				}
                break;

            case COM_PUT:
                if( fnEvidenceCollect( pGameState, *pParse ) )
				{
					pGameState->turnsLeft += ACT_PUT;
				}
                break;

            case COM_VIEW:
                if( fnEvidenceOrganize( pGameState, pParse ) )
				{
					pGameState->turnsLeft += ACT_VIEW;
				}
                break;

            case COM_LOOK:
				#if 0
				{
					printf("\nChar1: ");
					if (fnTepCharSearch(pGameState->liveList, pParse->Char1))
						printf("%s",printf(fnTepCharSearch(pGameState->liveList, pParse->Char1)->name));
					else
						printf("%08X", pParse->Char1);
					printf("\nChar2: ");
					if (fnTepCharSearch(pGameState->liveList, pParse->Char2))
						printf("%s",printf(fnTepCharSearch(pGameState->liveList, pParse->Char2)->name));
					else
						printf("%08X", pParse->Char2);
					printf("\nClue: %08X", pParse->Clue);
					printf("\nReserved: %08X", pParse->Reserved);
				}
				#endif
				/* if this if statement is true that means the player didn't enter in a valid thing to look at */
				/* so just have the fnDisplayPlayerLocation do a Display (have the player look at the floor) */
                if ( pParse->Clue == PARSE_UNUSED && pParse->Char1 == PARSE_UNUSED && pParse->Reserved == PARSE_UNUSED )
                {
                    pParse->Command = COM_DISPLAY;
					if( fnDisplayPlayerLocation( pGameState, *pParse ) )
					{
						/* fnDisplayPlayerLocation is now going to add ACT_GO to turnsLeft, lets undo that nate 3/26/01 */
						pGameState->turnsLeft -= ACT_GO;
					}
                }
                else
                {
					if( fnExamineObject( pGameState, pParse ) )
					{
						/* This is now done in fnExamineObject, nate 3/26/01 */
//						pGameState->turnsLeft += ACT_LOOK;
					}
                }


                break;

			case COM_SHOW:
				/* is the command correct? */
				if( pParse->Char1 != PARSE_UNUSED && pParse->Char2 != PARSE_UNUSED )
				{
					if(pParse->Char1 != ( TEP_CHARACTER | CHAR_SHEROESHA ))
					{
						if( fnShow( pGameState, *pParse ) )
						{
							pGameState->turnsLeft += ACT_SHOW;
						}
					}
					else
					{
						fnPrintOut(NOPAUSE, NULL,NULL,2, "\n", "Sheroesha, deep in meditation, fails to acknowledge you.\n");
					}
				}
				break;

			case COM_TALK:
				{
					if(pParse->Char1 != ( TEP_CHARACTER | CHAR_SHEROESHA ))
						fnInterpretTalk( pGameState, pParse->Char1 );
					else
						fnPrintOut(NOPAUSE, NULL,NULL,2, "\n", "Sheroesha, deep in meditation, fails to acknowledge you.\n");

					break;
				}

			case COM_GREET:
				if( pParse->Char1 != PARSE_UNUSED )
				{
					if(pParse->Char1 != ( TEP_CHARACTER | CHAR_SHEROESHA ))
					{
						if (pParse->Command != COM_GREET)
							printf("\n\n\n!@#$ERROR!@#$\n\n\n");
						if( fnGreet( pGameState, pParse->Char1 ) )
						{
							pGameState->turnsLeft += ACT_GREET;
						}
					}
					else
					{
						fnPrintOut(NOPAUSE, NULL,NULL,2, "\n", "Sheroesha, deep in meditation, fails to acknowledge you.\n");
					}
				}
				break;

			case COM_COMPLIMENT:
				if( pParse->Char1 != PARSE_UNUSED )
				{
					if(pParse->Char1 != ( TEP_CHARACTER | CHAR_SHEROESHA ))
					{
						if( fnCompliment( pGameState, pParse->Char1 ) )
						{
							pGameState->turnsLeft += ACT_COMPLIMENT;
						}
					}
					else
					{
						fnPrintOut(NOPAUSE, NULL,NULL,2, "\n", "Sheroesha, deep in meditation, fails to acknowledge you.\n");
					}
				}
				break;

			case COM_INSULT:
				if( pParse->Char1 != PARSE_UNUSED )
				{
					if(pParse->Char1 != ( TEP_CHARACTER | CHAR_SHEROESHA ))
					{
						if( fnInsult( pGameState, pParse->Char1 ) )
						{
							pGameState->turnsLeft += ACT_INSULT;
						}
					}
					else
					{
						fnPrintOut(NOPAUSE, NULL,NULL,2, "\n", "Sheroesha, deep in meditation, fails to acknowledge you.\n");
					}
				}
				break;

            case COM_LEAVE:
                pParse->Command = COM_NORTH;
                if( fnDisplayPlayerLocation( pGameState, *pParse ) )
				{
					pGameState->turnsLeft += ACT_LEAVE;
				}
                break;

			case COM_HELP:
				{
					TepParse msg;
					fnHelpFile();
					msg.Command = COM_DISPLAY;
					msg.Reserved = pGameState->currentRoom->id;

					fnDisplayPlayerLocation(pGameState, msg);

				break;
				}
			case COM_SAVE:
				/* save command of death!!! 
				{
					TepChar *pChar;
					TepIDUnNode *pKList;

					if( (pChar = fnTepCharSearch(pGameState->liveList, pParse->Char1) ) != NULL )
					{
						if( (pKList = fnTepIDSearch(pGameState->killersList, pChar->id) ) != NULL )
						{
							fnTepIDDelete(&pGameState->killersList, pKList);
							printf("%s is evil.\n", pChar->name);							
						}
						else
						{
							printf("%s is good.\n", pChar->name);
						}

						printf("You have killed %s dead.\n", pChar->name);
						fnTepWhackRemover( pGameState->liveList, pChar->id );
						fnTepCharDelete(&pGameState->liveList, pChar);						
					}
					else printf("Who do you want to kill... uh, save!\n");
				}
				break;*/
			case COM_LOAD:
				{
					fnTepFileManage(pGameState);
				}
				break;

            case COM_QUIT:
                printf("Are you sure you want to quit? (Y/N)"); //I added this so we could actually quit - Matt
                cChoice = fnGetInputChar();

                if( cChoice == 'Y' )
                    return IA_USER_QUIT;
                else
                    break;
			case COM_DONE:
				printf("Are you done investigating for the day? (Y/N)"); //I added this so we could actually quit - Matt
                cChoice = fnGetInputChar();
				
                if( cChoice == 'Y' )
                    return IA_END_OF_TURN;
                else
                    break;

        } /* End switch statement */


    } /* End while loop */

    return IA_END_OF_TURN;
}

BOOL    fnValidCommand( TepParse *pParse )
{
    switch ( pParse->Command )
    {
        case COM_PUT:
            /* If there's no clue, no good. */
            if ( !pParse->Clue )
            {
                printf( "You didn't specify a clue.\n" );
                return FALSE;
            }
            /* If we receive a -1, they don't have that clue */
            else if ( pParse->Clue == PARSE_UNUSED )		/*Changed -1 to PARSE_UNUSED NCC, LJC 2/13*/
            {
                printf( "You don't have that clue.\n" );
                return FALSE;
            }
            /* If the character's TepID is -1, no characters
               meet that name */
            if ( pParse->Char1 == PARSE_UNUSED )			/*Changed -1 to PARSE_UNUSED NCC, LJC 2/13*/
            {
                printf( "Who is that?\n" );
                return FALSE;
            }
            return TRUE;
        case COM_LOOK:

                printf( "What did you want to look at?\n" );

    }
    return TRUE;
}

BOOL fnIntActPrompt( TepCurState *pGameState, const char *pszAddOn )
{
    int     iHours      = 0,    /* The hours half of time (for clarity of code) */
            iMinutes    = 0;    /* The minutes half of time (same) */
	BOOL	iNightTime  = FALSE;    /* A flag (1/0) to determine if its past 12:59am */

	/* check for minutes over 59. increment the hour if it's needed */
	if( pGameState->turnsLeft % 100 >= 60 )
	{
		pGameState->turnsLeft += 40;
	}

    /* If the time is equal to or past 1 pm, subtract 12 hours and
       flip the iNightTime flag on  */
    if ( pGameState->turnsLeft >= 1200 )
    {
        iNightTime  = TRUE;
    }

    /* We've already checked for afternoon, now we determine hours 
		if the hour is zero (noon) then we set the hour to 12 */
    iHours      = pGameState->turnsLeft / 100;
	if( iHours > 12 )
		iHours -= 12 ;

    /* Determine the minutes */
    iMinutes    = ( pGameState->turnsLeft % 100 );//iTime - ( ( iTime / 100 ) * 100 );
                    /* total time - hours (as determined by modding, then remultiplying) */
                    /*      ex. 104 % 100 = 1    1 * 100 = 100   104 - 100 = 4 minutes   */

    printf("\n"); // just stuck in a new line...
    /* Print the hours, a colon, and the minutes */
    /* ##XXX */
	if( iHours != 0 )
		printf( "%2d", iHours );
	else
		printf( "%2d", iHours + 12 );

    /* ##:XX */
    printf( ":" );

    /* If the minutes are less than 10, we need to add an extra 0 */
    //if ( iMinutes < 10 )
    //    printf( "0" );

    /* ##:## */
    printf( "%02d", iMinutes );

    /* If the iNightTime flag is on, print PM.. */
    if ( iNightTime )
        printf( "PM" );
    /* otherwise print AM */
    else
        printf( "AM" );

	if (pszAddOn != NULL)
	{
		printf(" - %s",pszAddOn);
	}
    /* Print the prompt */
    printf( ">" );

	return TRUE;
}

/*
void main()
{
    TepCurState     GameState;
    pTepCurState    GameStatePtr;

    GameState.turnsLeft     = 935;
    GameStatePtr            = &GameState;

    fnInterpretAction( GameStatePtr );
}
*/
