/* header goes here...  version 0002 */
#include "tep_incs.h"




/**** TEP ID ************************************/

/*FUNCTION:     fnTepIDSearch
 *PARAMETERS:   TepIDUnNode             *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepIDUnNode *fnTepIDSearch(TepIDUnNode *pListRoot, TepID searchID)
{
    TepIDUnNode *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if((pCurrNode->NODE) == searchID)
        {
            return pCurrNode;
        }
    }
    return NULL;
}

/*FUNCTION:     fnTepIDAppend
 *PARAMETERS:   TepIDUnNode             **ppListRoot    Used as the starting point for the list
 *              TepID                   id              The TepId of the node
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends a node to a TepIDUnNode list.
*/
BOOL fnTepIDAppend(TepIDUnNode **ppListRoot, TepID NewID)
{
    TepIDUnNode *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepIDUnNode *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepIDUnNode));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->NODE = NewID;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepIDDelete
 *PARAMETERS:   TepIDUnNode             **pListRoot Root Pointer
 *              TepIDUnNode             *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepIDDelete(TepIDUnNode **ppListRoot, TepIDUnNode *pDeleteMe)
{
    TepIDUnNode *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepIDDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* free the memory */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}





/**** TEP DATA STRING ***************************/

/*FUNCTION:     fnTepIDSearch
 *PARAMETERS:   TepIDUnNode             *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepDataString *fnTepDataStringSearch(TepDataString *pListRoot, TepID searchID)
{
    TepDataString *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if((pCurrNode->id) == searchID)
        {
            return pCurrNode;
        }
    }
    return NULL;
}



/*FUNCTION:     fnTepDataStringAppend
 *PARAMETERS:   TepDataString           **ppListRoot, the first node in the string
 *              TepID                   DataStringID, TepID of the item
 *              pChar                   DataStringData, the data about the item.
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Links another DataString to the string of them
 *DESCRIPTION:  Works just like Matt's functions to link every thing together.
*/
BOOL fnTepDataStringAppend(TepDataString **ppListRoot, TepID DataStringID, pChar DataStringData)
{
    TepDataString *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepDataString *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepDataString));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        //printf("do we die here?");
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->id = DataStringID;
    pNewNode->data = DataStringData;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }
    return SUCCESS; /* YES! I won! */
}


/*FUNCTION:     fnTepDataStringDelete
 *PARAMETERS:   TepDataString                   **pListRoot Root Pointer
 *              TepPlayerEvidence               *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepDataStringDelete(TepDataString **ppListRoot, TepDataString *pDeleteMe)
{
    TepDataString *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepDataStringDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        //printf("-%x : ", pDeleteMe->data);

        /* free the data */
        if (pDeleteMe->data != NULL)
        	free( pDeleteMe->data ); //printf(".");

        /* free the memory */
        free(pDeleteMe); //printf(".");

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the data */
                if (pDeleteMe->data != NULL)
        			free(pDeleteMe->data);

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}





/**** TEP CONV DATA *****************************/

/*FUNCTION:     fnTepConvDataSearch
 *PARAMETERS:   TepConvData             *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepConvData *fnTepConvDataSearch(TepConvData *pListRoot, TepID searchID, TepID attachedto)
{
    TepConvData *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if( (pCurrNode->id & 0xffff0000) == searchID )
        {
            if( attachedto != 0 )
            {
                if( pCurrNode->attachedto == attachedto )
                    return pCurrNode;
            }
            else return pCurrNode;
        }
    }

    return NULL;
}

/*FUNCTION:     fnTepConvDataSearchRand
 *PARAMETERS:   TepConvData             *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepConvData *fnTepConvDataSearchRand(TepConvData *pListRoot, TepID searchID, TepID attachedto)
{
	int iX = 0;
	pTepConvData list[10] = {NULL};
	BOOL fDone = FALSE;

	while (iX < _sizeof(list) && !fDone )
	{
		if ((list[iX] = fnTepConvDataSearch(pListRoot, searchID, attachedto)) != NULL)
		{
			pListRoot = list[iX++]->NEXT;
		}
		else
		{
			/* there was no match so we are done */
			fDone = TRUE;
		}
	}

	/* check for zero results found - we don't want to mod by zero */
	if( iX == 0 )
	{
		return NULL;
	}

	/* return a random match */
	return list[rand() % iX];
	//return list[(iX-1) % rand()];
}

/*FUNCTION:     fnTepConvDataAppend
 *PARAMETERS:   TepConvData             **ppListRoot    Used as the starting point for the list
 *              TepID                   id              the nodes id
 *              TepID                   attachedTo      id of the character this is attached to
 *              pChar                   details         pointer to a string
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends a node to a TepConvData list.
*/
BOOL fnTepConvDataAppend(TepConvData **ppListRoot, TepID id, TepID attachedto, pChar details)
{
    TepConvData *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepConvData *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepConvData));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->id = id;
    pNewNode->attachedto = attachedto;
    pNewNode->details = details;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepConvDataDelete
 *PARAMETERS:   TepConvData             **pListRoot Root Pointer
 *              TepConvData             *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepConvDataDelete(TepConvData **ppListRoot, TepConvData *pDeleteMe)
{
    TepConvData *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepConvDataDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

		/* Jon - added to make sure we are not trying to free a null */
        /* first free the memory of attached strings */
		if (pDeleteMe->details != NULL)
			free(pDeleteMe->details);

        /* free the memory */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

				/* Jon - added to make sure we are not trying to free a null, and to delete the details which are malloced */
				/* first free the memory of attached strings */
				if (pDeleteMe->details != NULL)
					free(pDeleteMe->details);

				/* free the memory */
				free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}





/**** TEP ROOM **********************************/

/*FUNCTION:     fnTepRoomSearch
 *PARAMETERS:   TepRoom                 *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepRoom *fnTepRoomSearch(TepRoom *pListRoot, TepID searchID)
{
    TepRoom *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if((pCurrNode->id) == searchID)
        {
            return pCurrNode;
        }
    }
    return NULL;
}

/*FUNCTION:     fnTepRoomAppend
 *PARAMETERS:   TepRoom                 **ppListRoot    Used as the starting point for the list
 *              TepID                   id              the nodes id
 *              pChar                   name            string containing the room keyword
 *              pTepCmds                cmd             the commands allowed in the room
 *              pTepIDUnNode            rooms           pointer to a list of attached rooms
 *              pTepIDUnNode            objs            pointer to a list of attached objects
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends a node to a TepRoom list.
*/
BOOL fnTepRoomAppend(TepRoom **ppListRoot, TepID id, pChar name, pChar desc, TepCmds cmd, pTepIDUnNode rooms, pTepIDUnNode objs)
{
    TepRoom *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepRoom *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepRoom));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->id = id;
    pNewNode->name = name;
    pNewNode->desc = desc;
    pNewNode->cmd.allCmd = cmd.allCmd;
    pNewNode->rooms = rooms;
    pNewNode->objs = objs;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepRoomDelete
 *PARAMETERS:   TepRoom             **pListRoot Root Pointer
 *              TepRoom             *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepRoomDelete(TepRoom **ppListRoot, TepRoom *pDeleteMe)
{
    TepRoom *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepRoomDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        //printf(": %s - %x", pDeleteMe->name, pDeleteMe->name);

        /* first free the memory of attached strings */
        if (pDeleteMe->name != NULL)
	        free(pDeleteMe->name); //printf(".");
        if (pDeleteMe->desc != NULL)
	        free(pDeleteMe->desc); //printf(".");

        /* first free the memory of attached lists */
        fnTepIDDelete(&(pDeleteMe->rooms), NULL); //printf(".");
        fnTepIDDelete(&(pDeleteMe->objs), NULL); //printf(".");

        /* free the memory */
        free(pDeleteMe); //printf(". ");

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* first free the memory of attached strings */
		        if (pDeleteMe->name != NULL)
	                free(pDeleteMe->name);
		        if (pDeleteMe->desc != NULL)
	                free(pDeleteMe->desc);

                /* first free the memory of attached lists */
                fnTepIDDelete(&(pDeleteMe->rooms), NULL);
                fnTepIDDelete(&(pDeleteMe->objs), NULL);

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}






/**** TEP DYN ROOM ******************************/

/*FUNCTION:     fnTepDynRoomSearch
 *PARAMETERS:   TepDynRoom                  *pListRoot  Used as the starting point for the list search
 *              TepID                       searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepDynRoom *fnTepDynRoomSearch(TepDynRoom *pListRoot, TepID searchID)
{
    TepDynRoom *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
		#if defined(_DEBUG)
		//printf("..&%08X-i%08X-N%08X-n\"%s\"",pCurrNode, pCurrNode->asRoomID, pCurrNode->NEXT, pCurrNode->asRoom->name);
		#endif
        if((pCurrNode->asRoomID) == searchID)
        {
            return pCurrNode;
        }
        #if defined(_DEBUG)
        //printf("..\n");
        #endif
    }
    return NULL;
}

/*FUNCTION:     fnTepDynRoomAppend
 *PARAMETERS:   TepDynRoom                  **ppListRoot    Used as the starting point for the list
 *              TepID                   asRoomID        the associated rooms id
 *              pTepRoom                asRoom          the associated room node
 *              pTepIDUnNode            people          a list of people in this room
 *              pTepIDUnNode            objs            a list of objects in this room
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends a node to a TepRoom list.
*/
BOOL fnTepDynRoomAppend(TepDynRoom **ppListRoot, TepID asRoomID, pTepRoom asRoom, pTepIDUnNode people, pTepIDUnNode objs)
{
    TepDynRoom *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepDynRoom *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepDynRoom));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->asRoomID = asRoomID;
    pNewNode->asRoom = asRoom;
    pNewNode->people = people;
    pNewNode->objs = objs;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepDynRoomDelete
 *PARAMETERS:   TepDynRoom              **pListRoot Root Pointer
 *              TepDynRoom              *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepDynRoomDelete(TepDynRoom **ppListRoot, TepDynRoom *pDeleteMe)
{
    TepDynRoom *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepDynRoomDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* first free the memory of attached lists */
        fnTepIDDelete(&(pDeleteMe->people), NULL);
        fnTepIDDelete(&(pDeleteMe->objs), NULL);

        /* free the memory */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* first free the memory of attached lists */
                fnTepIDDelete(&(pDeleteMe->people), NULL);
                fnTepIDDelete(&(pDeleteMe->objs), NULL);

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}



/********************************************************************************************/
/*TepClueData list service functions*/

/*FUNCTION:     fnClueDataSearch
 *PARAMETERS:   TepPlayerEvidence       *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepClueData* fnClueDataSearch(TepClueData *pListRoot, TepID searchID, TepID attachedto)
{
    TepClueData *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if((pCurrNode->id & searchID) == searchID)
        {
			if( attachedto == 0 )
			{
				return pCurrNode;
			}
			else if( pCurrNode->attachedto == attachedto )
			{
				return pCurrNode;
			}
        }
    }
    return NULL;
}

/*FUNCTION:     fnClueDataAppend
 *PARAMETERS:   TepClueData             **ppListRoot    Used as the starting point for the list
 *              TepID                   id              The TepId of the clue data to append
 *              pTepUnNode              attachedto      pointer to a game object that the clue is attached to
 *              pChar                   details         string that contains a description of the clue
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends clue data to the clue data list.
*/
BOOL fnClueDataAppend(TepClueData **ppListRoot, TepID id, TepID attachedto, pChar details, pChar keyword)
{
    TepClueData *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepClueData *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepClueData));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->id = id;
    pNewNode->attachedto = attachedto;
    pNewNode->details = details;
	pNewNode->keyword = keyword;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnClueDataDelete
 *PARAMETERS:   TepClueData             **pListRoot Root Pointer
 *              TepClueData             *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnClueDataDelete(TepClueData **ppListRoot, TepClueData *pDeleteMe)
{
    TepClueData *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnClueDataDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* free the memory */
        if (pDeleteMe->details != NULL)
        	free(pDeleteMe->details);
        if (pDeleteMe->keyword != NULL)
        	free(pDeleteMe->keyword);
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the memory */
				if (pDeleteMe->details != NULL)
					free(pDeleteMe->details);
				if (pDeleteMe->keyword != NULL)
					free(pDeleteMe->keyword);
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}
/********************************************************************************************/
/*TepWhack list functions*/

/*FUNCTION:     fnTepWhackSearch
 *PARAMETERS:   TepWhack                *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepWhack* fnTepWhackSearch(TepWhack *pListRoot, TepID searchID)
{
    TepWhack *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if((pCurrNode->character) == searchID)
        {
            return pCurrNode;
        }
    }

    return NULL;
}

/*FUNCTION:     fnTepWhackAppend
 *PARAMETERS:   TepWhack                 **ppListRoot   Used as the starting point for the list
 *Struct Members added:
 *              TepID                   character
 *              USHORT                  value
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends clue data to the clue data list.
*/
BOOL fnTepWhackAppend(TepWhack **ppListRoot, TepID character, USHORT value)
{
    TepWhack *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepWhack *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepWhack));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->character = character;
    pNewNode->value = value;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepWhackDelete
 *PARAMETERS:   TepWhack                **pListRoot Root Pointer
 *              TepWhack                *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepWhackDelete(TepWhack **ppListRoot, TepWhack *pDeleteMe)
{
    TepWhack *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepWhackDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* free the memory */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}

/*FUNCTION:     fnTepWhackRemover
 *PARAMETERS:   TepChar *             *liveList   Used as the starting point for the list search
 *              TepID                 victimID    Remove this victim
 *RETURN VALUE: Whether the final operation was a success or not
 *SIDE EFFECTS: Removes the victim from the whacklist of everyone
 *DESCRIPTION:  This function removes the passed victimID from all live characters' whack list.
*/
BOOL fnTepWhackRemover(const pTepChar liveList, const TepID victimID)
{
	BOOL fRet = FAILURE;
	pTepChar pCurrentChar = liveList;

	while (pCurrentChar != NULL)
	{
		pTepWhack pWhack = fnTepWhackSearch(pCurrentChar->whacklist, victimID);

		fRet = fnTepWhackDelete(&pCurrentChar->whacklist, pWhack);

		pCurrentChar = pCurrentChar->NEXT;
	}

	return fRet;
}


/********************************************************************************************/
/*TepChar List Functions*/

/*FUNCTION:     fnTepCharSearch
 *PARAMETERS:   TepChar                 *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepChar* fnTepCharSearch(TepChar *pListRoot, TepID searchID)
{
    TepChar *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
		/*
			printf("&pCurrNode = %08X\n", &pCurrNode);
			printf("pCurrNode->id = %08X\n", pCurrNode->id);
			printf("pCurrNode->name = %s\n", pCurrNode->name);
			printf("pCurrNode->desc = %s\n", pCurrNode->desc);
			printf("pCurrNode->NEXT = %08X\n\n", pCurrNode->NEXT); */
        if((pCurrNode->id) == searchID)
        {
            return pCurrNode;
        }
    }

    return NULL;
}

/*FUNCTION:     fnTepCharAppend
 *PARAMETERS:   TepChar              **ppListRoot   Used as the starting point for the list
 *Struct Members added:
 *   TepID                   id                     unique id number of this character
 *   pChar                   name                   pointer to a null-termed string containing the character's name
 *   pChar                   desc                   pointer to a null-termed string containing the character's description
 *   pTepConvData           conv                   pointer to charcter's conversation matrix
 *   pTepWhack              whacklist              pointer to an bidirectional id list containing the character's whack data
 *   pTepClueData           clues                  pointer to a clue data structure containing actual attached clues are visible to the player upon description
 *   pTepIDUnNode            knownData              pointer to an unidirectional id list containing the data (clues, deaths and such) that the character knows about. This is used, so that the player cannot show a give character the same thing over and over to change their whack list
 *   pTepCharLikePlayer      like                   nibble containing levels of emotion toward the player
 *   pTepCharPersonality     ident                  bit field containing flags for personality traits
 *   pTepIDUnNode            locations              list of possible locations to find the character
 *   uChar                   complimentCounter      preset to a random number in each character at beginning of day, (between 1-3)
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends clue data to the clue data list.
*/
BOOL fnTepCharAppend(TepChar **ppListRoot, TepID id, pChar name, pChar desc, pTepConvData conv,
                   pTepWhack whackList, pTepIDUnNode clues, pTepIDUnNode knownData,
                   TepCharLikePlayer like, pTepCharPersonality ident, pTepIDUnNode locations,
                   uChar complimentCounter)
{
    TepChar *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepChar *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepChar));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->id = id;
    pNewNode->name = name;
    pNewNode->desc = desc;
    pNewNode->conv = conv;
    pNewNode->whacklist = whackList;
    pNewNode->clues = clues;
    pNewNode->knownData = knownData;
    pNewNode->like = like;
    pNewNode->ident = ident;
    pNewNode->locations = locations;
    pNewNode->complimentCounter = complimentCounter;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepCharDelete
 *PARAMETERS:   TepChar                 **pListRoot Root Pointer
 *              TepChar                 *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepCharDelete(TepChar **ppListRoot, TepChar *pDeleteMe)
{
    TepChar *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepCharDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* free the memory */
        /* what is my name? */
        if (pDeleteMe->name != NULL)
        	free(pDeleteMe->name);

        /* I don't know who I am */
        if (pDeleteMe->desc != NULL)
        	free(pDeleteMe->desc);

        /* I have nothing to say */
        fnTepConvDataDelete(&pDeleteMe->conv, NULL);

        /* I cannot harm anything */
        fnTepWhackDelete(&pDeleteMe->whacklist, NULL);

        /* I leave nothing behind */
        fnTepIDDelete(&pDeleteMe->clues, NULL);

        /* make me stupid */
        fnTepIDDelete(&pDeleteMe->knownData, NULL);

        /* free my mind */
        if (pDeleteMe->ident != NULL)
        	free(pDeleteMe->ident);

        /* I have nowhere to go */
        fnTepIDDelete(&pDeleteMe->locations, NULL);

        /* bye-bye! */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the memory */
				/* what is my name? */
				if (pDeleteMe->name != NULL)
					free(pDeleteMe->name);

				/* I don't know who I am */
				if (pDeleteMe->desc != NULL)
					free(pDeleteMe->desc);

				/* I have nothing to say */
				fnTepConvDataDelete(&pDeleteMe->conv, NULL);

				/* I cannot harm anything */
				fnTepWhackDelete(&pDeleteMe->whacklist, NULL);

				/* I leave nothing behind */
				fnTepIDDelete(&pDeleteMe->clues, NULL);

				/* make me stupid */
				fnTepIDDelete(&pDeleteMe->knownData, NULL);

				/* free my mind */
				if (pDeleteMe->ident != NULL)
					free(pDeleteMe->ident);

				/* I have nowhere to go */
				fnTepIDDelete(&pDeleteMe->locations, NULL);

				/* bye-bye! */
				free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}

/********************************************************************************************/
/*TepCurMurder List functions*/

/*FUNCTION:     fnTepCurMurderSearch
 *PARAMETERS:   TepCurMurder            *pListRoot  Used as the starting point for the list search
 *              pTepChar                searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepCurMurder* fnTepCurMurderSearch(TepCurMurder *pListRoot, TepID searchID)
{
    TepCurMurder *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if( (pCurrNode->death->id) == searchID)
        {
            return pCurrNode;
        }
    }

    return NULL;
}

/*FUNCTION:     fnTepCurMurderAppend
 *PARAMETERS:   TepCurMurder            **ppListRoot    Used as the starting point for the list
 *Struct Members added:
 *              pTepChar                victim
 *              pTepChar                murderer
 *              pTepDeath               death
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends clue data to the clue data list.
*/
BOOL fnTepCurMurderAppend(TepCurMurder **ppListRoot, TepID victim, TepID murderer, pTepDeath death)
{
    TepCurMurder *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepCurMurder *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepCurMurder));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->death = death;
    pNewNode->murderer = murderer;
    pNewNode->victim = victim;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }
    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepCurMurderDelete
 *PARAMETERS:   TepCurMurder                **pListRoot Root Pointer
 *              TepCurMurder                *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepCurMurderDelete(TepCurMurder **ppListRoot, TepCurMurder *pDeleteMe)
{
    TepCurMurder *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepCurMurderDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* free the death struct */
        //free( pDeleteMe->death );
		fnTepDeathDelete( &pDeleteMe->death, NULL );

        /* free the memory */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the death struct */
                //free( pDeleteMe->death );
				fnTepDeathDelete( &pDeleteMe->death, NULL );

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}
/********************************************************************************************/
/*TepPlayerEvidence lists*/

/*FUNCTION:     fnTepEvidenceAppend
 *PARAMETERS:   TepPlayerEvidenceBiNode **ppListRoot    Used as the starting point for the list
 *              TepID                   clueID          The id of the clue to append
 *              TepID                   charID          The id of the character the evidence is against. 0 for generic.
 *RETURN VALUE: 1 for Success
                0 for failure: Clue already in the list.
                -1 for failure: Unsuccessful malloc.
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends clues to evidence list. If the clue already exists then the clue
 *              will not be appended.
*/
BOOL fnTepEvidenceAppend(TepPlayerEvidence **ppListRoot, TepID clueID, TepID charID, TepID whereID)
{
    TepPlayerEvidence *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepPlayerEvidence *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepPlayerEvidence));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->clue = clueID;
    pNewNode->attachedTo = charID;
	pNewNode->whereFound = whereID;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}


/*FUNCTION:     fnTepEvidenceSearch
 *PARAMETERS:   TepPlayerEvidence       *pListRoot  Used as the starting point for the list search
 *              TepID                   clueID      Search for this parameter
 *              TepID                   charID      Search for this parameter
 *RETURN VALUE: pointer to a matching node or NULL.
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a clue in the list corresponding to the given node pointer.
*/
TepPlayerEvidence *fnTepEvidenceSearch(TepPlayerEvidence *pListRoot, TepID clueID, TepID charID)
{
    TepPlayerEvidence *pCurrNode;

	if( charID != 0 )
	{
		for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
		{
			if(((pCurrNode->clue) == clueID) && ((pCurrNode->attachedTo) == charID))
			{
				return pCurrNode;
			}
		}
	}
	else 
	{
		for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
		{
			if( pCurrNode->clue == clueID )
			{
				return pCurrNode;
			}
		}
	}
    return NULL;
}

/*FUNCTION:     fnTepEvidenceDelete
 *PARAMETERS:   TepPlayerEvidence               **pListRoot Root Pointer
 *              TepPlayerEvidence               *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepEvidenceDelete(TepPlayerEvidence **ppListRoot, TepPlayerEvidence *pDeleteMe)
{
    TepPlayerEvidence *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepEvidenceDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* free the memory */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}


/********************************************************************************************/
/*TepPList List functions*/

/*FUNCTION:     fnTepPListSearch
 *PARAMETERS:   TepPList                *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
/*
    THIS IS ONLY A SAMPLE, YOU WILL NEED TO WRITE A SIMILAR FUNCTION FOR YOUR
    SPECIFIC NEEDS.

TepPList *fnTepPListSearch(TepPList *pListRoot, TepID searchID)
{
    TepPList *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if((pCurrNode->NODE) == searchID)
        {
            return pCurrNode;
        }
    }
    return NULL;
}
*/

/*FUNCTION:     fnTepPListAppend
 *PARAMETERS:   TepPList                **ppListRoot    Used as the starting point for the list
 *              void                    *pNode          A pointer to something.
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends a node to a TepPList list.
*/
BOOL fnTepPListAppend(TepPList **ppListRoot, void *pNode)
{
    TepPList *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepPList *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepPList));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->pNode = pNode;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepPListDelete
 *PARAMETERS:   TepPList             **pListRoot Root Pointer
 *              TepPList             *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepPListDelete(TepPList **ppListRoot, TepPList *pDeleteMe)
{
    TepPList *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepPListDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

        /* free the memory */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

                /* free the memory */
                free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}





/**** TEP DEATH *********************************/

/*FUNCTION:     fnTepDeathSearch
 *PARAMETERS:   TepDeath                *pListRoot  Used as the starting point for the list search
 *              TepID                   searchID    Search for this parameter
 *RETURN VALUE: Pointer to the node in the list that the search matches
 *SIDE EFFECTS: None
 *DESCRIPTION:  This function searches for a node in the list corresponding to the given node pointer.
*/
TepDeath *fnTepDeathSearch( TepDeath *pListRoot, TepID searchID )
{
    TepDeath *pCurrNode;

    for(pCurrNode = pListRoot; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        if((pCurrNode->id) == searchID)
        {
            return pCurrNode;
        }
    }
    return NULL;
}

/*FUNCTION:     fnTepDeathAppend
 *PARAMETERS:   TepDeath             **ppListRoot    Used as the starting point for the list
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Adds a node and changes the appropriate pointers in the list structures.
 *DESCRIPTION:  This function appends a node to a TepDeath list.
*/
BOOL fnTepDeathAppend( TepDeath **ppListRoot, TepID id, TepID location, pChar desc, pTepConvData conv, pTepClueData clues )
{
    TepDeath *pCurrNode = *ppListRoot;   /* point to the first item in the list or NULL */
    TepDeath *pNewNode;                  /* pointer to the newly created node */

    /* allocate memory for the node */
    pNewNode = malloc(sizeof(TepDeath));

    /* if the malloc fails return failure */
    if(pNewNode == NULL)
    {
        return FAILURE;
    }

    /* if the memory was allocated successfully initialize the structs members */
    pNewNode->id = id;
	pNewNode->location = location;
	pNewNode->desc = desc;
	pNewNode->conv = conv;
	pNewNode->clues = clues;
    pNewNode->NEXT = NULL;

    /* if there is no first item in the list... */
    if(*ppListRoot == NULL)
    {
        /* put the new node there */
        *ppListRoot = pNewNode;
    }
    else
    {
        /* else loop to the end of the list */
        while(pCurrNode->NEXT != NULL)
        {
            pCurrNode = pCurrNode->NEXT;
        }

        /* place the new node at the end */
        pCurrNode->NEXT = pNewNode;
    }

    return SUCCESS; /* YES! I won! */
}

/*FUNCTION:     fnTepDeathDelete
 *PARAMETERS:   TepDeath                **pListRoot Root Pointer
 *              TepDeath	            *pDeleteMe  node to delete
 *RETURN VALUE: SUCCESS or FAILURE
 *SIDE EFFECTS: Deletes nodes or an entire list
 *DESCRIPTION:  Delets a node in a list or an entire list if NULL is passed to pDeleteMe
*/
BOOL fnTepDeathDelete( TepDeath **ppListRoot, TepDeath *pDeleteMe )
{
    TepDeath *pPrevNode, *pCurrNode;

    /* if there is no list... */
    if(*ppListRoot == NULL)
    {
        /* return failure, there's nothing to delete */
        return FAILURE;
    }
    /* if pDeleteMe is NULL then delete the entire list */
    else if(pDeleteMe == NULL)
    {
        while(*ppListRoot != NULL)
        {
            fnTepDeathDelete(ppListRoot, *ppListRoot);
        }
    }
    /* special check incase we need to delete the first item */
    else if(*ppListRoot == pDeleteMe)
    {
        /* close up the list around the dead node */
        *ppListRoot = pDeleteMe->NEXT;

		/* free up members */
		/* We don't know what it is. */
		if (pDeleteMe->desc != NULL)
			free( pDeleteMe->desc );

		/* We have nothing to say at this time */
		fnTepConvDataDelete( &pDeleteMe->conv, NULL );

		/* We cannot find anything on it */
		fnClueDataDelete( &pDeleteMe->clues, NULL );

        /* Therefore, it must not exist */
        free(pDeleteMe);

        /* this is a comment on success */
        return SUCCESS;
    }
    /* it's not the first element in the list so search for it */
    else
    {
        /* set pCurrNode to the beginning of the list */
        pCurrNode = *ppListRoot;

        /* loop through the list until we hit NULL */
        while(pCurrNode->NEXT != NULL)
        {
            /* increment the pointers */
            pPrevNode = pCurrNode;
            pCurrNode = pCurrNode->NEXT;

            /* is this the node we need to delete? */
            if(pCurrNode == pDeleteMe)
            {
                /* close up the list around the dead node */
                pPrevNode->NEXT = pCurrNode->NEXT;

				/* free up members */
				/* We don't know what it is. */
				if (pDeleteMe->desc != NULL)
					free( pDeleteMe->desc );

				/* We have nothing to say at this time */
				fnTepConvDataDelete( &pDeleteMe->conv, NULL );

				/* We cannot find anything on it */
				fnClueDataDelete( &pDeleteMe->clues, NULL );

				/* Therefore, it must not exist */
				free(pDeleteMe);

                /* this is a comment on success */
                return SUCCESS;
            }
        } /* end while */
    } /* end else - the catch all else */

    /* no match was found */
    return FAILURE;
}
