#include "tep_incs.h"





/* New Code */
/*  FUNCTION:		fnPrintOut
	PARAMETERS:		fPause,						Passing NOPAUSE will cause fnPrintOut to return as soon as all of its
												arguments have been formatted and printed to screen.  Passing PAUSE
												will cause fnPrintOut to pause the game after all of its arguments have
												been formatted and printed to screen; it will not return until the user
												has whacked any key.  If you OR in the MALE constant, then PrintOut will
												textually substitute the sex of the character as male (i.e. "he or his"
												rather than "she or her").  Otherwise, PrintOut treats the sex as female.

					clue						Pass a string that is the name of a clue; this string will replace any
												occurrence of '*' characters in the varargs.  Pass NULL to ignore this formatting.

					character					Pass a string that is the name of a character; this string will replace any
												occurrence of '@' characters in the varargs.  Pass NULL to ignore this formatting.

					in_values,					Pass the number of subsequent char * arguments.
	
					...							n number of char * arguments (varargs), each of which will be formatted and 
												printed to screen, in the order they are passed.

	RETURN VALUE:	0 (failure), or 1 (success)	
	SIDE EFFECTS:	See fPause.
	DESCRIPTION:	Takes a variable number of char *s and prints them to the screen verbatim, in the order that they
					are passed.  fnPrintOut formats text in the following manner:
					1) Makes sure no words near the right hand side of the screen are "cut in half".
					2) Makes sure that if more than one full screenful of text is to be printed at once, text printing
					stops at each screenful.  Printing resumes once the player strikes any key.
					3) Each '#' that is encountered is replaced with a paragraph indentation, i.e. a '\n' character
					followed by four adjacent ' ' characters.  Each '*' that is encountered is replaced with the clue 
					string.  Each '@' that is encountered is replaced with the character string.  Each '%' is replaced 
					with a "she or he" (as defined by the third bit in fPause); each '^' is replaced with "her or his"
					(as defined by the third bit in fPause).  
					See fPause for additional behaviour.  
					
					NOTE: Each special character ('#', '*', '@', '%', '^') requires a ' '
					character to precede it (i.e. "All your base are belong to us.#."  won't work.  
					But " #How are you gentlemen!!!" will function correctly).  

					Summary of reserved characters: ('#', '*', '@', '%', '^', '~').
 */

int fnPrintOut(int fPause, char *clue, char *character, int in_values, ...)
{
	int iChars = 0;				/* Counts the number of characters in a line. */
	int iLines = 0;				/* Counts the number of lines on the screen in one call to fnPrintOut. */
	int i = 0;					/* The index into the current string. */
	int n = 0;					/* A generic counter. */
	char cTempBuff[90] = {'~'};	/* The temporary memory buffer where each "chunk" of text is stored; it is terminated */
								/* with a '~' character and should never contain any ' ' characters. */

	va_list marker;				/* Marks the first of the variable arguments. */
	char *arg;					/* Contains the current argument.  */
	int count = 0;				/* A counter that keeps track of the number of variable arguments that have been */
								/* processed. */

	/** Following lines were added by nate 4/11/01, just in case somebody passes a null for clue or character **/
	char IfNullClue[2] = "*";
	char IfNullCharacter[2] = "@";

	if(clue == NULL)
	{
		clue = IfNullClue;
	}

	if(character == NULL)
	{
		character = IfNullCharacter;
	}
	/**********************************************************************************************************/

	/* If fPause was passed unmeaningful values... 
	if ( (fPause != PAUSE) && (fPause != NOPAUSE) )
	{*/
		/* Then complain about it and return 0 for failure. 
		printf("fnPrintOut is complaining that fPause, it's first argument, was passed unmeaningful values!\n");
		return 0;
	}*/

	va_start(marker, in_values);		/* Begin using variable arguments. */
	arg = va_arg(marker, char *);		/* Initialize arg with the first variable argument. */

	/* Process every variable argument.  After an argument has been processed, intialize arg with the next */
	/* variable argument; also set i to 0 so that it corresponds to the first character of the new variable argument. */
	for (; count < in_values; arg = va_arg(marker, char *), ++count, i = 0)
	{

		/* Until the terminating NULL is reached in the string... */
		while (*(arg + i) != 0)
		{
			/* If a '#' character is reached... */
			if (*(arg + i) == '#')
			{
				++i;					/* Advance to the next character in the argument (without printing the '#') */
				printf("\n    ");		/* Print a paragraph indentation */
				iChars = 4;				/* Update iChars to reflect that we are now 4 spaces into the line. */
				++iLines;				/* Update the iLines to reflect that we are now one line down from before. */
				continue;				/* and return to the top of the loop to test if we have hit the NULL yet. */
			} /* end if */

			/* Place every character (save '#', ' ', '~' and NULL) in the string, one after another, into the temporary 
			buffer. */
			for (n = 0; *(arg + i) != ' ' && *(arg + i) != 0; ++n, ++i)
			{
				/* Check for the '*' special character; if one is encountered, deal with it accordingly. */
				if (*(arg + i) == '*')
				{
					int iSpecialCount;	/* A generic counter */

					++i;				/* Advance to the next character in the argument (without printing the '*') */
					for (iSpecialCount = 0; iSpecialCount < (int)strlen(clue); ++iSpecialCount, ++n)
					{
						cTempBuff[n] = *(clue + iSpecialCount);		/* "Inject" the clue name into the text output (using the */
																	/* temporary buffer as the vector) character by character. */
					}/* end for loop */

				}/* end if */
				
				/* Check for the '%' special character; if one is encountered, deal with it accordingly. */
				if (*(arg + i) == '%')
				{
					int iSpecialCount;	/* A generic counter */
					++i;				/* Advance to the next character in the argument (without printing the '*') */

					if (fPause & MALE)
					{
						char *gender = "he";

						for (iSpecialCount = 0; iSpecialCount < 2; ++iSpecialCount, ++n)
						{
							cTempBuff[n] = *(gender + iSpecialCount);/* "Inject" the word into the text output (using the */
							/* temporary buffer as the vector) character by character. */
						}/* end for loop */
					}
					else
					{
						char *gender = "she";
						
						for (iSpecialCount = 0; iSpecialCount < 3; ++iSpecialCount, ++n)
						{
							cTempBuff[n] = *(gender + iSpecialCount);/* "Inject" the word into the text output (using the */
							/* temporary buffer as the vector) character by character. */
						}/* end for loop */
					}
				}/* end if */

				/* Check for the '^' special character; if one is encountered, deal with it accordingly. */
				if (*(arg + i) == '^')
				{
					int iSpecialCount;	/* A generic counter */
					++i;				/* Advance to the next character in the argument (without printing the '*') */

					if (fPause & MALE)
					{
						char *gender = "his";

						for (iSpecialCount = 0; iSpecialCount < 3; ++iSpecialCount, ++n)
						{
							cTempBuff[n] = *(gender + iSpecialCount);/* "Inject" the word into the text output (using the */
							/* temporary buffer as the vector) character by character. */
						}/* end for loop */
					}
					else
					{
						char *gender = "her";
						
						for (iSpecialCount = 0; iSpecialCount < 3; ++iSpecialCount, ++n)
						{
							cTempBuff[n] = *(gender + iSpecialCount);/* "Inject" the word into the text output (using the */
							/* temporary buffer as the vector) character by character. */
						}/* end for loop */
					}
				}/* end if */

				/* Check for the '@' special character; if one is encountered, deal with it accordingly. */
				if (*(arg + i) == '@')
				{
					int iSpecialCount;	/* A generic counter */

					++i;				/* Advance to the next character in the argument (without printing the '@') */
					for (iSpecialCount = 0; iSpecialCount < (int)strlen(character); ++iSpecialCount, ++n)
					{
						cTempBuff[n] = *(character + iSpecialCount);/* "Inject" the clue name into the text output (using the */
																	/* temporary buffer as the vector) character by character. */
					}/* end for loop */
				}/* end if */

				cTempBuff[n] = *(arg + i);
				cTempBuff[n + 1] = '~';				/* A delimiting '~' will always terminate the temporary buffer */
	
				/* If the number of characters in the temporary buffer exceeds the number of remaining characters */
				/* on the line... */
				if (n > (76 - iChars) )
				{
					printf("\n");	/* Jump down to the next line... */
					++iLines;		/* Increment iLines to keep track of the number of lines remaining onscreen. */
					iChars = 0;		/* Reset the number of remaining characters on this line to zero */
				} /* end if */

				/* If the number of lines onscreen in one fnPrintOut exceeds 21... */
				if (iLines > 21)
				{					
					while(!_kbhit())					/* Pause the game until the player whacks any key. */						
					{
						;
					} /* end while loop */							
					_getch();							/* Throw away the character the player entered when */
														/* he or she whacked said any key. */

					iLines = 0;							/* Reset iLines to zero, as we are now on a fresh screen. */
				} /* end if */
			} /* end for loop */

			/* If the current character is a ' ' or a NULL... */
			if (*(arg + i) == ' ' || *(arg + i) == 0)
			{	
				/* Until the terminating '~' character is reached... */
				for (n = 0; cTempBuff[n] != '~'; ++n, ++iChars)
				{
					printf("%c", cTempBuff[n]);			/* Print every character in the temporary buffer.  For each */
														/* character printed, increment iChars, to keep track of how */
														/* much space remains on the line. */
				} /* end for loop */

				cTempBuff[0] = '~';	/* Make sure the '~' character is present to denote that there is nothing to print */
									/* in the temporary buffer anymore. */

				/* If the current character is indeed a ' ' and not a NULL... */
				if (*(arg + i) == ' ')
				{
					++iChars;			/* Increment iChars once, for the ' ' character. */
					++i;				/* Increment to the next character in the string. */
					printf(" ");		/* and finally print the ' '. */
				} /* end if */
			} /* end if */
		} /* end while loop */

	} /* end for loop */
	
	va_end (marker);					/* We are finished using varargs. */

	if (fPause == PAUSE)
	{
					
		while(!_kbhit())								/* Pause the game until the player whacks any key. */				
		{
			;
		} /* end while loop */								
		_getch();										/* Throw away the character the player entered when */
														/* he or she whacked any key. */
	} /* end if */

	return 1;

}




/* Old Code... */
#if 0
/*  FUNCTION:       fnPrintOut
    PARAMETERS:     fPause,                     Passing NOPAUSE will cause fnPrintOut to return as soon as all of its
                                                arguments have been formatted and printed to screen.  Passing PAUSE
                                                will cause fnPrintOut to pause the game after all of its arguments have
                                                been formatted and printed to screen; it will not return until the user
                                                has whacked any key.
                    in_values,                  Pass the number of subsequent char * arguments.
                    ...                         n number of char * arguments, each of which will be formatted and
                                                printed to screen, in the order they are passed.
    RETURN VALUE:   0 (failure), or 1 (success)
    SIDE EFFECTS:   See fPause.
    DESCRIPTION:    Takes a variable number of char *s and prints them to the screen verbatim, in the order that they
                    are passed.  fnPrintOut formats text in the following manner:
                    1) Makes sure no words near the right hand side of the screen are "cut in half".
                    2) Makes sure that if more than one full screenful of text is to be printed at once, text printing
                    stops at each screenful.  Printing resumes once the player strikes any key.
                    3) Each ' #' (that's ' ' followed by a '#') that is encountered is replaced with a paragraph 
					indentation, i.e. a '\n' character followed by four adjacent ' ' characters.
                    See fPause for additional behaviour.
 */
int fnPrintOut(int fPause, int in_values, ...)
{
    int iChars = 0;             /* Counts the number of characters in a line. */
    int iLines = 0;             /* Counts the number of lines on the screen in one call to fnPrintOut. */
    int i = 0;                  /* The index into the current string. */
    int n = 0;                  /* A generic counter. */
    char cTempBuff[31] = {'~'}; /* The temporary memory buffer where each "chunk" of text is stored; it is terminated */
                                /* with a '~' character and should never contain any ' ' characters. */

    va_list marker;             /* Marks the first of the variable arguments. */
    char *arg;                  /* Contains the current argument.  */
    int count = 0;              /* A counter that keeps track of the number of variable arguments that have been */
                                /* processed. */

    /* If fPause was passed unmeaningful values... */
    if ( (fPause != PAUSE) && (fPause != NOPAUSE) )
    {
        /* Then complain about it and return 0 for failure. */
        printf("fnPrintOut is complaining that fPause, it's first argument, was passed unmeaningful values!\n");
        return 0;
    }

    va_start(marker, in_values);        /* Begin using variable arguments. */
    arg = va_arg(marker, char *);       /* Initialize arg with the first variable argument. */

    /* Process every variable argument.  After an argument has been processed, intialize arg with the next */
    /* variable argument; also set i to 0 so that it corresponds to the first character of the new variable argument. */
    for (; count < in_values; arg = va_arg(marker, char *), ++count, i = 0)
    {

        /* Until the terminating NULL is reached in the string... */
        while (*(arg + i) != 0) // 0 was NULL
        {
            /* If a '#' character is reached... */
            if (*(arg + i) == '#')
            {
                ++i;                    /* Advance to the next character (without printing the '#') */
                printf("\n    ");       /* Print a paragraph indentation */
                iChars = 4;             /* Update iChars to reflect that we are now 4 spaces into the line. */
                ++iLines;               /* Update the iLines to reflect that we are now one line down from before. */
                continue;               /* and return to the top of the loop to test if we have hit the NULL yet. */
            } /* end if */

			if (*(arg + i) == '@')
            {
                ++i;                    /* Advance to the next character (without printing the '#') */
                printf("\n    ");       /* Print a paragraph indentation */
                iChars = 4;             /* Update iChars to reflect that we are now 4 spaces into the line. */
                ++iLines;               /* Update the iLines to reflect that we are now one line down from before. */
                continue;               /* and return to the top of the loop to test if we have hit the NULL yet. */
            } /* end if */

            /* Place every character (save '#', ' ', '~' and NULL) in the string, one after another, into the temporary
            buffer. */
            for (n = 0; *(arg + i) != ' ' && *(arg + i) != 0; ++n, ++i) // 0 was NULL
            {
                cTempBuff[n] = *(arg + i);
                cTempBuff[n + 1] = '~';             /* A delimiting '~' will always terminate the temporary buffer */

                /* If the number of characters in the temporary buffer exceeds the number of remaining characters */
                /* on the line... */
                if (n > (76 - iChars) )
                {
                    printf("\n");   /* Jump down to the next line... */
                    ++iLines;       /* Increment iLines to keep track of the number of lines remaining onscreen. */
                    iChars = 0;     /* Reset the number of remaining characters on this line to zero */
                } /* end if */

                /* If the number of lines onscreen in one fnPrintOut exceeds 21... */
                if (iLines > 21)
                {
                    while(!_kbhit())                    /* Pause the game until the player whacks any key. */
                    {
                        ;
                    } /* end while loop */
                    _getch();                           /* Throw away the character the player entered when */
                                                        /* he or she whacked said any key. */

                    iLines = 0;                         /* Reset iLines to zero, as we are now on a fresh screen. */
                } /* end if */
            } /* end for loop */

            /* If the current character is a ' ' or a NULL... */
            if (*(arg + i) == ' ' || *(arg + i) == 0) // 0 was NULL
            {
                /* Until the terminating '~' character is reached... */
                for (n = 0; cTempBuff[n] != '~'; ++n, ++iChars)
                {
                    printf("%c", cTempBuff[n]);         /* Print every character in the temporary buffer.  For each */
                                                        /* character printed, increment iChars, to keep track of how */
                                                        /* much space remain on the line. */
                } /* end for loop */

                cTempBuff[0] = '~'; /* Make sure the '~' character is present to denote that there is nothing to print */
                                    /* in the temporary buffer anymore. */

                /* If the current character is indeed a ' ' and not a NULL... */
                if (*(arg + i) == ' ')
                {
                    ++iChars;           /* Increment iChars once, for the ' ' character. */
                    ++i;                /* Increment to the next character in the string. */
                    printf(" ");        /* and finally print the ' '. */
                } /* end if */
            } /* end if */
        } /* end while loop */

    } /* end for loop */

    va_end (marker);                    /* We are finished using varargs. */

    if (fPause == PAUSE)
    {

        while(!_kbhit())                                /* Pause the game until the player whacks any key. */
        {
            ;
        } /* end while loop */
        _getch();                                       /* Throw away the character the player entered when */
                                                        /* he or she whacked any key. */
    } /* end if */

    return 1;

}
#endif