#include "tep_incs.h"

/*FUNCTION:     fnSacrificeTime
 *PARAMETERS:   TepCurState *State  pointer to the game state
 *RETURN VALUES:SUCCESS or FAILURE
 *SIDE EFFECTS: Prints out actions sequences and character reactions
 *              deletes the current days sacrifice from the game state
 *DESCRIPTION:  This function does the sacrifice for the day.
 *              -Displays intro dialogue
 *              -Displays each player's accusation for the day
 *              -Displays the sacrifice scene
 *              -Displays the characters running down the hill
 *              -Displays the statue crumbling or not
 *              -Displays players' reactions to the above
 *              -Displays the player retiring for the day
 */
BOOL fnSacrificeTime(TepCurState *State)
{
    TepChar         *pCurrNode = NULL;      /*Pointer for searching*/
    TepChar	    *pDiffNode = NULL;	    /*A different node for searching*/
    TepChar         *pChar = NULL;          /*Ditto*/
	TepChar			*pCharWhack = NULL;		/*Ditto*/
    TepWhack        whackMe;                /*Comparison whack struct*/
    TepWhack        *pVWhack = NULL;        /*Village whackList*/
    TepWhack        *pVWhackStart = NULL;   /*Pointer to start of whack list*/
    TepConvData     *pSacrifice = NULL;     /*Pointer Sacrifice Conv. data*/
    TepConvData     *pIntroSpeech = NULL;   /*Pointer to general intro*/
    TepConvData     *pClosingSpeech = NULL; /*Pointer to general closing*/
    TepConvData     *pSpeech = NULL;        /*Pointer to conve data*/
    TepConvData     *pHillRun = NULL;       /*Pointer to an action sequence*/
    TepIDUnNode     *pKiller = NULL;        /*Pointer for the killer list*/
    int             i;                      /*generic counter*/
	BOOL			CheckGameOver;
	char			KillerNumber[2] = {0};	/* set string to null, added by nate 4/11/01 */
	TepIDUnNode		*IDKillerList = NULL;	/* added by nate 4/11/01 */

    /*Create a list and keep a pointer to the beginning*/
    fnTepWhackAppend(&pVWhack, 0, 0);
    pVWhackStart = pVWhack;

    /*End of day intro*/
    if((pIntroSpeech = fnTepConvDataSearch(State->genConv, TEP_TALK | TALK_EODINTRO, 0)) == NULL)
    {
		printf("Did not find EOD speech\n");
        return FAILURE;
    }
    fnPrintOut(PAUSE, NULL,NULL,1, pIntroSpeech->details);

    /*Search through the list of living characters to determine who has the highest whack value*/
    for(pCurrNode = State->liveList; pCurrNode != NULL; pCurrNode = pCurrNode->NEXT)
    {
        TepWhack *pCurrent;
        TepWhack *pVictim;

		if( (pCurrNode->id & CHAR_SHEROESHA) == CHAR_SHEROESHA )
			continue;

        /*Initialize whackMe to the first whack struct in the current character's whack list*/
        whackMe.character = pCurrNode->whacklist->character;
        whackMe.value = pCurrNode->whacklist->value;

        /*Search through the current character's whack list to find who's got the highest value*/
        for(pCurrent = pCurrNode->whacklist; pCurrent != NULL; pCurrent = pCurrent->NEXT)
        {
            if(pCurrent->value > whackMe.value)
            {
                whackMe.character = pCurrent->character;
                whackMe.value = pCurrent->value;
            }
        }

        /*Character's Speech as to who to whack goes here*/
        for(pSpeech = pCurrNode->conv; pSpeech != NULL; pSpeech = pSpeech->NEXT)
        {
            if(((pSpeech->id & TALK_EODACCUSATION)==TALK_EODACCUSATION) && (pSpeech->attachedto == whackMe.character))
            {
                printf("\n\n%s steps forward and says: ", pCurrNode->name);
                fnPrintOut(PAUSE, NULL,NULL,1, pSpeech->details);

                break;
            }
        }
	printf("\n");

	/*Find the accused and adjust the accused's suspicion of the accuser*/
	for(pDiffNode = State->liveList; pDiffNode != NULL; pDiffNode = pDiffNode->NEXT)
	{
		if(pDiffNode->id == whackMe.character)
		{
			TepWhack *pBlah;
				for(pBlah = pDiffNode->whacklist; pBlah != NULL; pBlah = pBlah->NEXT)
				{
					if(pBlah->character == pCurrNode->id)
					{
						pBlah->value+=5;
					}
				}
		}
	}

        /*Append the the current character's high whack character onto the village whacklist*/
        if((pVictim = fnTepWhackSearch(pVWhack, whackMe.character)) == NULL)
        {
            fnTepWhackAppend(&pVWhack, whackMe.character, 1);
        }
        else
        {
            /*If the victim is already in the whacklist, increment the counter*/
            pVictim->value++;
        }
    } /*End for*/

    /*Reuse whackMe for the new Comparisons*/
    whackMe.character = pVWhackStart->character;
    whackMe.value = pVWhackStart->value;

    /*Search through the newly created Village whack list to see who has the most votes*/
    for(pVWhack = pVWhackStart; pVWhack != NULL; pVWhack = pVWhack->NEXT)
    {
        if(pVWhack->value > whackMe.value)
        {
            whackMe.character = pVWhack->character;
            whackMe.value = pVWhack->value;
        }
    } /*end for*/

    /*Gain access to the whackee's data*/
    if((pCharWhack = fnTepCharSearch(State->liveList, whackMe.character)) == NULL)
    {
		printf("\nSac. Time Character not found\n");
        return FAILURE;
    }

    /*Let Shoeresha point to the whackee*/
    printf("\n\nThe villagers look to Shoeresha to choose the sacrifice from their accusations.\n");
    printf("Shoeresha raises her hand and points at %s.\n", pCharWhack->name);

    /*Description of Sacrifice goes here*/
    pSacrifice = fnTepConvDataSearch(pCharWhack->conv, TEP_TALK | TALK_SACRIFICE, 0);
	if(pSacrifice == NULL)
	{
		printf("\nSacrifice data not found.");
		return FAILURE;
	}

    fnPrintOut(PAUSE, NULL,NULL,1, pSacrifice->details);

	/******************************** Added by Nate 4/11/01  ********************************/
	/* find out how many statues are left by counting how many killers are still alive */
	for(i = 0,	IDKillerList = State->killersList; IDKillerList != NULL; IDKillerList = IDKillerList->NEXT)
	{
		if(fnTepCharSearch(State->liveList, IDKillerList->NODE) != NULL)
			i++;		
	}

	/* convert that number to a string so we know how many statues there we're before the killing */
	_itoa(i, KillerNumber, 10);
	/**************************************************************************************/

	/* remove the dead character from character's whack lists */
	fnTepWhackRemover( State->liveList, pCharWhack->id );

    /*Remove Character from the live list*/
    fnTepCharDelete(&State->liveList, pCharWhack);

    /*Description of running down the hill goes here*/
    if((pHillRun = fnTepConvDataSearch(State->genConv, TEP_TALK | TALK_EODHILLRUN, 0)) == NULL)
    {
		printf("\nVillagers running down hill data not found.");
        return FAILURE;
    }

	/* pass that number to fnPrintOut, print out the hill run stuff */
	fnPrintOut(PAUSE, KillerNumber,NULL,1, pHillRun->details);

	/*Statue crumbling goes here*/
	if((pKiller = fnTepIDSearch(State->killersList, whackMe.character)) == NULL)
	{
		/*Statue does not crumble*/
		if((pSpeech = fnTepConvDataSearch(State->genConv, TEP_TALK | TALK_EODNOCRUMBLE, 0)) == NULL)
		{
			printf("\nConversation search resulted in error.");
			return FAILURE;
		}
		else
		{
			fnPrintOut(PAUSE, NULL,NULL,1, pSpeech->details);
		}

		/* test to check if the  game is over */
		if( ( CheckGameOver = fnGameOverCheck( State ) ) != PLAYER_CONT )
		{
			fnGameOver( State, CheckGameOver );
			return IA_USER_QUIT;
		}
		else
		{
			/*Villagers sad reactions*/
			for((pChar = State->liveList), i = 0; (pChar != NULL) && (i < 4); (pChar = pChar->NEXT), i++)
			{
				TepConvData *pReaction = NULL;

				if( pChar->id == (TEP_CHARACTER | CHAR_SHEROESHA) )
				{
					break;
				}

				for(pReaction = pChar->conv; pReaction != NULL; pReaction = pReaction->NEXT)
				{
					if(((pReaction->id & TALK_EODSADSPEECH) == TALK_EODSADSPEECH) && (pReaction->attachedto == whackMe.character))
					{
						switch(rand() % 4)
						{
						case 0:
							printf("\n\n%s tearfully says: ", pChar->name);
							break;
						case 1:
							printf("\n\n%s cries out: ", pChar->name);
							break;
						case 2:
							printf("\n\n%s laments: ", pChar->name);
							break;
						case 3:
							printf("\n\n%s shouts in despair: ", pChar->name);
							break;
						}
						fnPrintOut(PAUSE, NULL,NULL,1, pReaction->details);
						break;
					}
				}
				if(pReaction == NULL)
				{
					printf("\nReaction was not found for %s.", pChar->name);
					return FAILURE;
				}
			}
		}
	}
	else
	{
		/*Statue crumbles*/
		if((pSpeech = fnTepConvDataSearch(State->genConv, TEP_TALK | TALK_EODCRUMBLE, 0)) == NULL)
		{
			printf("\nConversation search resulted in error.");
			return FAILURE;
		}
		else
		{
			fnPrintOut(PAUSE, NULL,NULL,1, pSpeech->details);
		}

		/* delete the killer's id from the killer's list */
		if(fnTepIDDelete(&State->killersList, pKiller) == FAILURE)
		{
			printf("\nDeletion was unsuccessful.");
			return FAILURE;
		}

		/* test to check if the  game is over */
		if( ( CheckGameOver = fnGameOverCheck( State ) ) != PLAYER_CONT )
		{
			fnGameOver( State, CheckGameOver );
			return IA_USER_QUIT;
		}
		else
		{
			/*Villager's positive reactions*/
			for((pChar = State->liveList), i = 0; (pChar != NULL) && (i < 4); (pChar = pChar->NEXT), i++)
			{
				TepConvData *pReaction = NULL;

				if( pChar->id == (TEP_CHARACTER | CHAR_SHEROESHA) )
				{
					break;
				}

				for(pReaction = pChar->conv; pReaction != NULL; pReaction = pReaction->NEXT)
				{
					if(((pReaction->id & TALK_EODHAPPYSPEECH) == TALK_EODHAPPYSPEECH) && (pReaction->attachedto == whackMe.character))
					{
						printf("\n\n%s says: ", pChar->name);
						fnPrintOut(PAUSE, NULL,NULL,1, pReaction->details);
						break;
					}
				}
				if(pReaction == NULL)
				{
					printf("\nReaction was not found for %s.", pChar->name);
					return FAILURE;
				}
			}
		}
	}

	/*End of day intro*/
	if((pClosingSpeech = fnTepConvDataSearch(State->genConv, TEP_TALK | TALK_EODCLOSINGSPEECH, 0)) == NULL)
	{
		printf("\nClosing speech data was not found.");
		return FAILURE;
	}
	printf("\n\n");
	fnPrintOut(PAUSE, NULL,NULL,1, pClosingSpeech->details);
	printf("\n\n");


    return IA_END_OF_TURN;
}