#include "tep_incs.h"

/* fnFindTheStinkinMurder finds the murder by searching the dynmap and all of it's object lists
	returns a pointer to the dynmap node */
TepDynRoom *fnFindTheStinkinMurder( TepCurState *pGameState, TepID MID )
{
	TepDynRoom *pCurRoom = pGameState->mapLite;

	for( ; pCurRoom != NULL; pCurRoom = pCurRoom->NEXT )
	{
		TepIDUnNode *pDeath = NULL;

		/* search for the death in the object list */
		if( ( pDeath = fnTepIDSearch( pCurRoom->objs, MID ) ) != NULL )
		{
			/* this is the place... */
			return pCurRoom;
		}
	}

	return NULL;
}

/* fnSheroesha's clue is essentially one massive "Glaxian Case Statement" that associates room
	IDs with region or room specific clues that allude to the murder's location */
BOOL fnSheroeshaClue( TepCurState *gameState )
{
	TepID roomID = 0;
	TepDynRoom *murderLocationP = NULL;
	TepConvData *theClueP = NULL;						/* pointer to today's clue */
	TepCurMurder *murderP = gameState->currentMurder;	/* pointer to a murder, init to the murderlist root */

	/* get the locationID of the current day's murder */
	if( murderP != NULL )
	{
		for( ; murderP->NEXT != NULL; murderP = murderP->NEXT );
	}

	/* get the location of the murder if it's not loc specific */
	if( murderP->death->location == 0 )
	{
		murderLocationP = fnFindTheStinkinMurder( gameState, murderP->death->id );
		if( murderLocationP == NULL )
			return FALSE;
		roomID = murderLocationP->asRoomID;
	}
	else
	{
		roomID = murderP->death->location;
	}

	/* Determine what clue to generate, based on where today's murder has occurred */
	switch( roomID )
	{
		/* For huts */
	case 0x04010017:
	case 0x04010018:
	case 0x0401001c:
	case 0x04010019:
	case 0x0401001a:
	case 0x0401001d:
	case 0x0401001b:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_PERSONALCLUE, 0 );
			break;
		}
		/* For abandoned longhouses */
	case 0x04010021:
	case 0x04010020:
	case 0x04010023:
	case 0x04010022:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_ABANDONEDCLUE, 0 );
			break;
		}
		/* For beaches */
	case 0x04010001:
	case 0x04010002:
	case 0x04010003:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_SANDYCLUE, 0 );
			break;
		}
		/* For forest paths or bean patch */
	case 0x04010011:
	case 0x04010010:
	case 0x0401000f:
	case 0x04010004:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_FOLIAGECLUE, 0 );
			break;
		}
		/* For cliffs, descending cliff path or bridge */
	case 0x04010006:
	case 0x04010007:
	case 0x0401000c:
	case 0x04010008:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_RAVINECLUE, 0 );
			break;
		}
		/* For coral reefs */
	case 0x0401000a:
	case 0x0401000b:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_CORALCLUE, 0 );
			break;
		}
		/* For working areas */
	case 0x04010015:
	case 0x04010009:
	case 0x04010036:
	case 0x04010037:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_WORKCLUE, 0 );
			break;
		}
		/* For communal places (villages and eating hut) */
	case 0x04010031:
	case 0x0401000d:
	case 0x04010032:
	case 0x04010034:
	case 0x04010033:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_COMMUNALCLUE, 0 );
			break;
		}
		/* For watery places */
	case 0x04010005:
	case 0x0401001e:
	case 0x0401001f:
	case 0x0401002d:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_WATERCLUE, 0 );
			break;
		}
		/* For hot areas */
	case 0x04010012:
	case 0x04010016:
	case 0x04010035:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_WARMTHCLUE, 0 );
			break;
		}
		/* For graveyards */
	case 0x04010013:
	case 0x04010014:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_GRAVECLUE, 0 );
			break;
		}
		/* For marshes */
	case 0x04010028:
	case 0x04010027:
	case 0x04010026:
	case 0x0401002b:
	case 0x0401002a:
	case 0x04010029:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_SWAMPCLUE, 0 );
			break;
		}
		/* For kiddie places */
	case 0x04010024:
	case 0x04010025:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_CHILDCLUE, 0 );
			break;
		}
		/* For swampy beach */
	case 0x0401002e: 
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_SWAMPSANDCLUE, 0 );
			break;
		}
		/* For Temple of Rametep */
	case 0x0401002f:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_DARKCLUE, 0 );
			break;
		}
		/* Plains clue */
	case 0x0401002c:
	case 0x0401000e:
		{
			theClueP = fnTepConvDataSearch( gameState->genConv, TEP_TALK | TALK_PLAINSCLUE, 0 );
			break;
		}
	}
	
	/* if we found a matching clue print it out */
	if( theClueP != NULL )
	{
		fnPrintOut( PAUSE, NULL, NULL, 1, theClueP->details );
	}

	return TRUE;
}

/*
SHEROESHACLUE	1963	CONV_PERSONALCLUE	CHAR_GEN	#Sheroeshas sightless, white eyes snap open.  "I sense that a horrible atrocity has been committed in a villager of Teps personal living space!"  Just as quickly, Sheroeshas eyes snap shut.
SHEROESHACLUE	1964	CONV_ABANDONEDCLUE	CHAR_GEN	#Though Sheroeshas form is just as unmoving as always, you can hear a soundless voice in your head.  A GREAT EVIL HAS BEEN DONE IN AN ABANDONED LIVING SPACE OF TEP.  BEGIN YOUR SEARCH FOR THE PERPETRATOR.
SHEROESHACLUE	1965	CONV_SANDYCLUE	CHAR_GEN	#Though Sheroesha sits stock still, her lips move almost imperceptibly.  "Someone has been murdered in a place full of sand," she says.
SHEROESHACLUE	1966	CONV_FOLIAGECLUE	CHAR_GEN	#Sheroeshas eyelids flutter as she whispers under her breath, "A place full of foliage has been defiled by an act of horrible violence."  Her eyelids become abruptly still.
SHEROESHACLUE	1967	CONV_RAVINECLUE	CHAR_GEN	#Sheroeshas decrepit, reclining form catches your eye.  A moment ago, you were sure she was merely sitting crosslegged on the ground.  Now she is floating half a foot in the air!  A deep, baritone voice rumbles forth from her ancient, cracked lips: "Malevolence has spilled over and expressed itself near a dangerous, rocky ravine."  The masculine, sonorous voice echoes in the Temple as Sheroesha descends and touches down to the ground beneath her.
SHEROESHACLUE	1968	CONV_CORALCLUE	CHAR_GEN	#Sheroesha speaks in sharp, clipped tones. "Someones life has been ended near a place full of bright coral."  She speaks no more.
SHEROESHACLUE	1969	CONV_WORKCLUE	CHAR_GEN	#Shereosha speaks.  "My labours in the astral plane have revealed to me that someone has been killed in a place of work; a place where my kin toil to create the things of everyday life."  The venerable Speaker of the Sun sounds vastly fatigued.  "Please localize your search in such areas."  Sheroesha becomes silent. 
SHEROESHACLUE	1970	CONV_COMMUNALCLUE	CHAR_GEN	#The thin, white wisps of Sheroeshas hair are standing on end, as though repelled from one another by static electricity.  "Outsider!"  Her voice rasps sharply.  "An awful maliciousness has been committed in a communal area of Tep."  It sounds as though phlegm catches in her throat, and then the Speaker of the Sun speaks no more.
SHEROESHACLUE	1971	CONV_WATERCLUE	CHAR_GEN	#Sheroeshas body is as motionless as ever, but a gleam, a sharp reflection of light off of her face attracts your eye.  As you look closer, you notice that a tear is sliding down one of her ancient cheeks.  The tear is joined by more of its brethren, and fluid is soon pouring out of Sheroeshas half-closed, all-white eyes.  In a calm, unemotional voice, she intones, "A watery place has borne witness to a horrible act.  Begin your search."  A single tear of blood snakes dribbles from one eyelid, snakes its way over the bridge of her sharp, avian nose, and falls to the ground.
SHEROESHACLUE	1972	CONV_WARMTHCLUE	CHAR_GEN	#Sheroesha opens her eyes and seems to stare into your very soul with a pair of deep, brown irises and clear, immaculate pupils.  In a lilting, feminine voice (that could not possibly belong to one as aged as she) she addresses you.  "Outsider, a place where Tepititans Warmth bubbles forth from the Earth is where you must begin your search.  The awful deed was effected in a place such as that."  As you look on, astounded, Sheroeshas perfect eyes fade back into a pure white, and she takes on an ancient appearance once more.  
SHEROESHACLUE	1973	CONV_GRAVECLUE	CHAR_GEN	#Sheroesha utters in a flat, emotionless voice: "The Place of the Dead has a new resident.  Choose this place as the locus for your search."  A single eyelash drifts from one of Sheroeshas eyelid, floating to the ground.
SHEROESHACLUE	1974	CONV_SWAMPCLUE	CHAR_GEN	#You notice that Sheroeshas body is trembling.  The trembling becomes more pronounced, until her body seems to be wracked with spasms.  Her ancient voice is raised to a hideous shriek: "The place where Our Fathers land is choked with water, sickness and disease...the reeking bog...that is where the evil has taken place!!"  Abruptly, Sheroeshas shaking ceases, and she slumps on her side.
SHEROESHACLUE	1975	CONV_CHILDCLUE	CHAR_GEN	#You turn around to see Sheroesha standing, erect, before you.  Smoothly approaching you, she places her leathery hands on both of your shoulders, and stares at you with her pure, white eyes.  "Someone has been murdered in a place of the childrens.  Begin your investigation."  Sheroeshas hands fall by her sides, and she turns on her heel before taking her position on the temple ground once again.  
SHEROESHACLUE	1976	CONV_SWAMPSANDCLUE	CHAR_GEN	#You notice that Sheroesha is muttering to herself.  Drawing nearer, you hear her repeating the phrase, "Where the swamp and the sand meet".  
SHEROESHACLUE	1977	CONV_DARKCLUE	CHAR_GEN	#In a gravelly, exhausted voice, Sheroesha says, "An unholy, secret area of Tep is the site of Rameteps murder.  Near the waterfall, the portal to... Abruptly, Sheroeshas throat constricts with an audible hacking noise, and she is silent.
SHEROESHACLUE	1978	CONV_PLAINSCLUE	CHAR_GEN	#Sheroesha speaks softly.  "One of Tepititans faithful has passed away in a flat place full of grass."  She grimaces, but says no more.
*/
